from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.position_book import PositionBook


T = TypeVar("T", bound="GetInstrumentsInstrumentPositionBookResponse200")


@_attrs_define
class GetInstrumentsInstrumentPositionBookResponse200:
    """
    Attributes:
        position_book (Union[Unset, PositionBook]): The representation of an instrument's position book at a point in
            time
    """

    position_book: Union[Unset, "PositionBook"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        position_book: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.position_book, Unset):
            position_book = self.position_book.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if position_book is not UNSET:
            field_dict["positionBook"] = position_book

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.position_book import PositionBook

        d = src_dict.copy()
        _position_book = d.pop("positionBook", UNSET)
        position_book: Union[Unset, PositionBook]
        if isinstance(_position_book, Unset):
            position_book = UNSET
        else:
            position_book = PositionBook.from_dict(_position_book)

        get_instruments_instrument_position_book_response_200 = cls(
            position_book=position_book,
        )

        get_instruments_instrument_position_book_response_200.additional_properties = d
        return get_instruments_instrument_position_book_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .fixed_price_order_position_fill import FixedPriceOrderPositionFill
from .fixed_price_order_position_fill import check_fixed_price_order_position_fill
from .fixed_price_order_state import FixedPriceOrderState
from .fixed_price_order_state import check_fixed_price_order_state
from .fixed_price_order_type import FixedPriceOrderType
from .fixed_price_order_type import check_fixed_price_order_type
from .stop_loss_details import StopLossDetails
from .take_profit_details import TakeProfitDetails
from .trailing_stop_loss_details import TrailingStopLossDetails
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="FixedPriceOrder")


@dataclasses.dataclass
class FixedPriceOrder:
    """A FixedPriceOrder is an order that is filled immediately upon creation using a fixed price.

    Attributes:
        id (Optional[str]): The Order's identifier, unique within the Order's Account.
        create_time (Optional[str]): The time when the Order was created.
        state (Optional[FixedPriceOrderState]): The current state of the Order.
        client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        type (Optional[FixedPriceOrderType]): The type of the Order. Always set to "FIXED_PRICE" for Fixed Price
            Orders.
        instrument (Optional[str]): The Fixed Price Order's Instrument.
        units (Optional[str]): The quantity requested to be filled by the Fixed Price Order. A posititive number of
            units results in a long Order, and a negative number of units results in a short Order.
        price (Optional[str]): The price specified for the Fixed Price Order. This price is the exact price that the
            Fixed Price Order will be filled at.
        position_fill (Optional[FixedPriceOrderPositionFill]): Specification of how Positions in the Account are
            modified when the Order is filled.
        trade_state (Optional[str]): The state that the trade resulting from the Fixed Price Order should be set to.
        take_profit_on_fill (Optional[TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
            Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
            a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss_on_fill (Optional[StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
            Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss_on_fill (Optional[TrailingStopLossDetails]): TrailingStopLossDetails specifies the
            details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
            filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
            is modified directly through the Trade.
        trade_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        filling_transaction_id (Optional[str]): ID of the Transaction that filled this Order (only provided when the
            Order's state is FILLED)
        filled_time (Optional[str]): Date/time when the Order was filled (only provided when the Order's state is
            FILLED)
        trade_opened_id (Optional[str]): Trade ID of Trade opened when the Order was filled (only provided when the
            Order's state is FILLED and a Trade was opened as a result of the fill)
        trade_reduced_id (Optional[str]): Trade ID of Trade reduced when the Order was filled (only provided when
            the Order's state is FILLED and a Trade was reduced as a result of the fill)
        trade_closed_i_ds (Optional[List[str]]): Trade IDs of Trades closed when the Order was filled (only provided
            when the Order's state is FILLED and one or more Trades were closed as a result of the fill)
        cancelling_transaction_id (Optional[str]): ID of the Transaction that cancelled the Order (only provided
            when the Order's state is CANCELLED)
        cancelled_time (Optional[str]): Date/time when the Order was cancelled (only provided when the state of the
            Order is CANCELLED)"""

    id: Optional[str]
    create_time: Optional[str]
    state: Optional[FixedPriceOrderState]
    client_extensions: Optional["ClientExtensions"]
    type: Optional[FixedPriceOrderType]
    instrument: Optional[str]
    units: Optional[str]
    price: Optional[str]
    position_fill: Optional[FixedPriceOrderPositionFill]
    trade_state: Optional[str]
    take_profit_on_fill: Optional["TakeProfitDetails"]
    stop_loss_on_fill: Optional["StopLossDetails"]
    trailing_stop_loss_on_fill: Optional["TrailingStopLossDetails"]
    trade_client_extensions: Optional["ClientExtensions"]
    filling_transaction_id: Optional[str]
    filled_time: Optional[str]
    trade_opened_id: Optional[str]
    trade_reduced_id: Optional[str]
    trade_closed_i_ds: Optional[List[str]]
    cancelling_transaction_id: Optional[str]
    cancelled_time: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .take_profit_details import TakeProfitDetails
        from .trailing_stop_loss_details import TrailingStopLossDetails
        from .client_extensions import ClientExtensions
        from .stop_loss_details import StopLossDetails

        d = src_dict.copy()
        id = d.pop("id", None)
        create_time = d.pop("createTime", None)
        _state = d.pop("state", None)
        state: Optional[FixedPriceOrderState]
        if _state is None:
            state = None
        else:
            state = check_fixed_price_order_state(_state)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _type = d.pop("type", None)
        type: Optional[FixedPriceOrderType]
        if _type is None:
            type = None
        else:
            type = check_fixed_price_order_type(_type)
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        price = d.pop("price", None)
        _position_fill = d.pop("positionFill", None)
        position_fill: Optional[FixedPriceOrderPositionFill]
        if _position_fill is None:
            position_fill = None
        else:
            position_fill = check_fixed_price_order_position_fill(_position_fill)
        trade_state = d.pop("tradeState", None)
        _take_profit_on_fill = d.pop("takeProfitOnFill", None)
        take_profit_on_fill: Optional[TakeProfitDetails]
        if _take_profit_on_fill is None:
            take_profit_on_fill = None
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)
        _stop_loss_on_fill = d.pop("stopLossOnFill", None)
        stop_loss_on_fill: Optional[StopLossDetails]
        if _stop_loss_on_fill is None:
            stop_loss_on_fill = None
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)
        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", None)
        trailing_stop_loss_on_fill: Optional[TrailingStopLossDetails]
        if _trailing_stop_loss_on_fill is None:
            trailing_stop_loss_on_fill = None
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )
        _trade_client_extensions = d.pop("tradeClientExtensions", None)
        trade_client_extensions: Optional[ClientExtensions]
        if _trade_client_extensions is None:
            trade_client_extensions = None
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )
        filling_transaction_id = d.pop("fillingTransactionID", None)
        filled_time = d.pop("filledTime", None)
        trade_opened_id = d.pop("tradeOpenedID", None)
        trade_reduced_id = d.pop("tradeReducedID", None)
        trade_closed_i_ds = cast(List[str], d.pop("tradeClosedIDs", None))
        cancelling_transaction_id = d.pop("cancellingTransactionID", None)
        cancelled_time = d.pop("cancelledTime", None)
        fixed_price_order = cls(
            id=id,
            create_time=create_time,
            state=state,
            client_extensions=client_extensions,
            type=type,
            instrument=instrument,
            units=units,
            price=price,
            position_fill=position_fill,
            trade_state=trade_state,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
            filling_transaction_id=filling_transaction_id,
            filled_time=filled_time,
            trade_opened_id=trade_opened_id,
            trade_reduced_id=trade_reduced_id,
            trade_closed_i_ds=trade_closed_i_ds,
            cancelling_transaction_id=cancelling_transaction_id,
            cancelled_time=cancelled_time,
        )
        fixed_price_order.additional_properties = d
        return fixed_price_order

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="TradeReduce")


@dataclasses.dataclass
class TradeReduce:
    """A TradeReduce object represents a Trade for an instrument that was reduced (either partially or fully) in an
    Account. It is found embedded in Transactions that affect the position of an instrument in the account, specifically
    the OrderFill Transaction.

        Attributes:
            trade_id (Optional[str]): The ID of the Trade that was reduced or closed
            units (Optional[str]): The number of units that the Trade was reduced by
            price (Optional[str]): The average price that the units were closed at. This price may be clamped for
                guaranteed Stop Loss Orders.
            realized_pl (Optional[str]): The PL realized when reducing the Trade
            financing (Optional[str]): The financing paid/collected when reducing the Trade
            guaranteed_execution_fee (Optional[str]): This is the fee that is charged for closing the Trade if it has a
                guaranteed Stop Loss Order attached to it.
            half_spread_cost (Optional[str]): The half spread cost for the trade reduce/close. This can be a positive or
                negative value and is represented in the home currency of the Account."""

    trade_id: Optional[str]
    units: Optional[str]
    price: Optional[str]
    realized_pl: Optional[str]
    financing: Optional[str]
    guaranteed_execution_fee: Optional[str]
    half_spread_cost: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        trade_id = d.pop("tradeID", None)
        units = d.pop("units", None)
        price = d.pop("price", None)
        realized_pl = d.pop("realizedPL", None)
        financing = d.pop("financing", None)
        guaranteed_execution_fee = d.pop("guaranteedExecutionFee", None)
        half_spread_cost = d.pop("halfSpreadCost", None)
        trade_reduce = cls(
            trade_id=trade_id,
            units=units,
            price=price,
            realized_pl=realized_pl,
            financing=financing,
            guaranteed_execution_fee=guaranteed_execution_fee,
            half_spread_cost=half_spread_cost,
        )
        trade_reduce.additional_properties = d
        return trade_reduce

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="CalculatedPositionState")


@dataclasses.dataclass
class CalculatedPositionState:
    """The dynamic (calculated) state of a Position

    Attributes:
        instrument (Optional[str]): The Position's Instrument.
        net_unrealized_pl (Optional[str]): The Position's net unrealized profit/loss
        long_unrealized_pl (Optional[str]): The unrealized profit/loss of the Position's long open Trades
        short_unrealized_pl (Optional[str]): The unrealized profit/loss of the Position's short open Trades
        margin_used (Optional[str]): Margin currently used by the Position."""

    instrument: Optional[str]
    net_unrealized_pl: Optional[str]
    long_unrealized_pl: Optional[str]
    short_unrealized_pl: Optional[str]
    margin_used: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        instrument = d.pop("instrument", None)
        net_unrealized_pl = d.pop("netUnrealizedPL", None)
        long_unrealized_pl = d.pop("longUnrealizedPL", None)
        short_unrealized_pl = d.pop("shortUnrealizedPL", None)
        margin_used = d.pop("marginUsed", None)
        calculated_position_state = cls(
            instrument=instrument,
            net_unrealized_pl=net_unrealized_pl,
            long_unrealized_pl=long_unrealized_pl,
            short_unrealized_pl=short_unrealized_pl,
            margin_used=margin_used,
        )
        calculated_position_state.additional_properties = d
        return calculated_position_state

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .delayed_trade_closure_transaction_reason import (
    DelayedTradeClosureTransactionReason,
)
from .delayed_trade_closure_transaction_reason import (
    check_delayed_trade_closure_transaction_reason,
)
from .delayed_trade_closure_transaction_type import DelayedTradeClosureTransactionType
from .delayed_trade_closure_transaction_type import (
    check_delayed_trade_closure_transaction_type,
)
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="DelayedTradeClosureTransaction")


@dataclasses.dataclass
class DelayedTradeClosureTransaction:
    """A DelayedTradeClosure Transaction is created administratively to indicate open trades that should have been closed
    but weren't because the open trades' instruments were untradeable at the time. Open trades listed in this
    transaction will be closed once their respective instruments become tradeable.

        Attributes:
            id (Optional[str]): The Transaction's Identifier.
            time (Optional[str]): The date/time when the Transaction was created.
            user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
            account_id (Optional[str]): The ID of the Account the Transaction was created for.
            batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
                batch are applied to the Account simultaneously.
            request_id (Optional[str]): The Request ID of the request which generated the transaction.
            type (Optional[DelayedTradeClosureTransactionType]): The Type of the Transaction. Always set to
                "DELAYED_TRADE_CLOSURE" for an DelayedTradeClosureTransaction.
            reason (Optional[DelayedTradeClosureTransactionReason]): The reason for the delayed trade closure
            trade_i_ds (Optional[str]): List of Trade ID's identifying the open trades that will be closed when their
                respective instruments become tradeable"""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[DelayedTradeClosureTransactionType]
    reason: Optional[DelayedTradeClosureTransactionReason]
    trade_i_ds: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[DelayedTradeClosureTransactionType]
        if _type is None:
            type = None
        else:
            type = check_delayed_trade_closure_transaction_type(_type)
        _reason = d.pop("reason", None)
        reason: Optional[DelayedTradeClosureTransactionReason]
        if _reason is None:
            reason = None
        else:
            reason = check_delayed_trade_closure_transaction_reason(_reason)
        trade_i_ds = d.pop("tradeIDs", None)
        delayed_trade_closure_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            reason=reason,
            trade_i_ds=trade_i_ds,
        )
        delayed_trade_closure_transaction.additional_properties = d
        return delayed_trade_closure_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .units_available_details import UnitsAvailableDetails
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="UnitsAvailable")


@dataclasses.dataclass
class UnitsAvailable:
    """Representation of how many units of an Instrument are available to be traded by an Order depending on its
    postionFill option.

        Attributes:
            default (Optional[UnitsAvailableDetails]): Representation of many units of an Instrument are available to be
                traded for both long and short Orders.
            reduce_first (Optional[UnitsAvailableDetails]): Representation of many units of an Instrument are available
                to be traded for both long and short Orders.
            reduce_only (Optional[UnitsAvailableDetails]): Representation of many units of an Instrument are available
                to be traded for both long and short Orders.
            open_only (Optional[UnitsAvailableDetails]): Representation of many units of an Instrument are available to
                be traded for both long and short Orders."""

    default: Optional["UnitsAvailableDetails"]
    reduce_first: Optional["UnitsAvailableDetails"]
    reduce_only: Optional["UnitsAvailableDetails"]
    open_only: Optional["UnitsAvailableDetails"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .units_available_details import UnitsAvailableDetails

        d = src_dict.copy()
        _default = d.pop("default", None)
        default: Optional[UnitsAvailableDetails]
        if _default is None:
            default = None
        else:
            default = UnitsAvailableDetails.from_dict(_default)
        _reduce_first = d.pop("reduceFirst", None)
        reduce_first: Optional[UnitsAvailableDetails]
        if _reduce_first is None:
            reduce_first = None
        else:
            reduce_first = UnitsAvailableDetails.from_dict(_reduce_first)
        _reduce_only = d.pop("reduceOnly", None)
        reduce_only: Optional[UnitsAvailableDetails]
        if _reduce_only is None:
            reduce_only = None
        else:
            reduce_only = UnitsAvailableDetails.from_dict(_reduce_only)
        _open_only = d.pop("openOnly", None)
        open_only: Optional[UnitsAvailableDetails]
        if _open_only is None:
            open_only = None
        else:
            open_only = UnitsAvailableDetails.from_dict(_open_only)
        units_available = cls(
            default=default,
            reduce_first=reduce_first,
            reduce_only=reduce_only,
            open_only=open_only,
        )
        units_available.additional_properties = d
        return units_available

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .position_book import PositionBook
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetInstrumentsInstrumentPositionBookResponse200")


@dataclasses.dataclass
class GetInstrumentsInstrumentPositionBookResponse200:
    """Attributes:
    position_book (Optional[PositionBook]): The representation of an instrument's position book at a point in
        time"""

    position_book: Optional["PositionBook"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .position_book import PositionBook

        d = src_dict.copy()
        _position_book = d.pop("positionBook", None)
        position_book: Optional[PositionBook]
        if _position_book is None:
            position_book = None
        else:
            position_book = PositionBook.from_dict(_position_book)
        get_instruments_instrument_position_book_response_200 = cls(
            position_book=position_book
        )
        get_instruments_instrument_position_book_response_200.additional_properties = d
        return get_instruments_instrument_position_book_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

OrderCancelReason = Literal[
    "ACCOUNT_LOCKED",
    "ACCOUNT_NEW_POSITIONS_LOCKED",
    "ACCOUNT_ORDER_CREATION_LOCKED",
    "ACCOUNT_ORDER_FILL_LOCKED",
    "ACCOUNT_POSITION_VALUE_LIMIT_EXCEEDED",
    "BOUNDS_VIOLATION",
    "CLIENT_REQUEST",
    "CLIENT_REQUEST_REPLACED",
    "CLIENT_TRADE_ID_ALREADY_EXISTS",
    "FIFO_VIOLATION",
    "HEDGING_GSLO_VIOLATION",
    "INSTRUMENT_ASK_HALTED",
    "INSTRUMENT_ASK_REDUCE_ONLY",
    "INSTRUMENT_BID_HALTED",
    "INSTRUMENT_BID_REDUCE_ONLY",
    "INSUFFICIENT_LIQUIDITY",
    "INSUFFICIENT_MARGIN",
    "INTERNAL_SERVER_ERROR",
    "LINKED_TRADE_CLOSED",
    "LOSING_TAKE_PROFIT",
    "MARKET_HALTED",
    "MIGRATION",
    "OPEN_TRADES_ALLOWED_EXCEEDED",
    "PENDING_ORDERS_ALLOWED_EXCEEDED",
    "POSITION_CLOSEOUT_FAILED",
    "POSITION_SIZE_EXCEEDED",
    "STOP_LOSS_ON_FILL_CLIENT_ORDER_ID_ALREADY_EXISTS",
    "STOP_LOSS_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "STOP_LOSS_ON_FILL_GUARANTEED_ASK_HALTED",
    "STOP_LOSS_ON_FILL_GUARANTEED_BID_HALTED",
    "STOP_LOSS_ON_FILL_GUARANTEED_HEDGING_NOT_ALLOWED",
    "STOP_LOSS_ON_FILL_GUARANTEED_LEVEL_RESTRICTION_EXCEEDED",
    "STOP_LOSS_ON_FILL_GUARANTEED_MINIMUM_DISTANCE_NOT_MET",
    "STOP_LOSS_ON_FILL_GUARANTEED_NOT_ALLOWED",
    "STOP_LOSS_ON_FILL_GUARANTEED_REQUIRED",
    "STOP_LOSS_ON_FILL_LOSS",
    "STOP_LOSS_ON_FILL_PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "STOP_LOSS_ON_FILL_REQUIRED",
    "STOP_LOSS_ON_FILL_TIME_IN_FORCE_INVALID",
    "STOP_LOSS_ON_FILL_TRIGGER_CONDITION_INVALID",
    "TAKE_PROFIT_ON_FILL_CLIENT_ORDER_ID_ALREADY_EXISTS",
    "TAKE_PROFIT_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "TAKE_PROFIT_ON_FILL_LOSS",
    "TAKE_PROFIT_ON_FILL_PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "TIME_IN_FORCE_EXPIRED",
    "TRAILING_STOP_LOSS_ON_FILL_CLIENT_ORDER_ID_ALREADY_EXISTS",
    "TRAILING_STOP_LOSS_ON_FILL_GTD_TIMESTAMP_IN_PAST",
]

ORDER_CANCEL_REASON_VALUES: Set[OrderCancelReason] = {
    "ACCOUNT_LOCKED",
    "ACCOUNT_NEW_POSITIONS_LOCKED",
    "ACCOUNT_ORDER_CREATION_LOCKED",
    "ACCOUNT_ORDER_FILL_LOCKED",
    "ACCOUNT_POSITION_VALUE_LIMIT_EXCEEDED",
    "BOUNDS_VIOLATION",
    "CLIENT_REQUEST",
    "CLIENT_REQUEST_REPLACED",
    "CLIENT_TRADE_ID_ALREADY_EXISTS",
    "FIFO_VIOLATION",
    "HEDGING_GSLO_VIOLATION",
    "INSTRUMENT_ASK_HALTED",
    "INSTRUMENT_ASK_REDUCE_ONLY",
    "INSTRUMENT_BID_HALTED",
    "INSTRUMENT_BID_REDUCE_ONLY",
    "INSUFFICIENT_LIQUIDITY",
    "INSUFFICIENT_MARGIN",
    "INTERNAL_SERVER_ERROR",
    "LINKED_TRADE_CLOSED",
    "LOSING_TAKE_PROFIT",
    "MARKET_HALTED",
    "MIGRATION",
    "OPEN_TRADES_ALLOWED_EXCEEDED",
    "PENDING_ORDERS_ALLOWED_EXCEEDED",
    "POSITION_CLOSEOUT_FAILED",
    "POSITION_SIZE_EXCEEDED",
    "STOP_LOSS_ON_FILL_CLIENT_ORDER_ID_ALREADY_EXISTS",
    "STOP_LOSS_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "STOP_LOSS_ON_FILL_GUARANTEED_ASK_HALTED",
    "STOP_LOSS_ON_FILL_GUARANTEED_BID_HALTED",
    "STOP_LOSS_ON_FILL_GUARANTEED_HEDGING_NOT_ALLOWED",
    "STOP_LOSS_ON_FILL_GUARANTEED_LEVEL_RESTRICTION_EXCEEDED",
    "STOP_LOSS_ON_FILL_GUARANTEED_MINIMUM_DISTANCE_NOT_MET",
    "STOP_LOSS_ON_FILL_GUARANTEED_NOT_ALLOWED",
    "STOP_LOSS_ON_FILL_GUARANTEED_REQUIRED",
    "STOP_LOSS_ON_FILL_LOSS",
    "STOP_LOSS_ON_FILL_PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "STOP_LOSS_ON_FILL_REQUIRED",
    "STOP_LOSS_ON_FILL_TIME_IN_FORCE_INVALID",
    "STOP_LOSS_ON_FILL_TRIGGER_CONDITION_INVALID",
    "TAKE_PROFIT_ON_FILL_CLIENT_ORDER_ID_ALREADY_EXISTS",
    "TAKE_PROFIT_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "TAKE_PROFIT_ON_FILL_LOSS",
    "TAKE_PROFIT_ON_FILL_PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "TIME_IN_FORCE_EXPIRED",
    "TRAILING_STOP_LOSS_ON_FILL_CLIENT_ORDER_ID_ALREADY_EXISTS",
    "TRAILING_STOP_LOSS_ON_FILL_GTD_TIMESTAMP_IN_PAST",
}


def check_order_cancel_reason(value: str) -> OrderCancelReason:
    if value in ORDER_CANCEL_REASON_VALUES:
        return cast(OrderCancelReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {ORDER_CANCEL_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="OpenTradeFinancing")


@_attrs_define
class OpenTradeFinancing:
    """OpenTradeFinancing is used to pay/collect daily financing charge for an open Trade within an Account

    Attributes:
        trade_id (Union[Unset, str]): The ID of the Trade that financing is being paid/collected for.
        financing (Union[Unset, str]): The amount of financing paid/collected for the Trade.
    """

    trade_id: Union[Unset, str] = UNSET
    financing: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        trade_id = self.trade_id

        financing = self.financing

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if trade_id is not UNSET:
            field_dict["tradeID"] = trade_id
        if financing is not UNSET:
            field_dict["financing"] = financing

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        trade_id = d.pop("tradeID", UNSET)

        financing = d.pop("financing", UNSET)

        open_trade_financing = cls(
            trade_id=trade_id,
            financing=financing,
        )

        open_trade_financing.additional_properties = d
        return open_trade_financing

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

TrailingStopLossOrderRequestType = Literal[
    "FIXED_PRICE",
    "LIMIT",
    "MARKET",
    "MARKET_IF_TOUCHED",
    "STOP",
    "STOP_LOSS",
    "TAKE_PROFIT",
    "TRAILING_STOP_LOSS",
]
TRAILING_STOP_LOSS_ORDER_REQUEST_TYPE_VALUES: Set[TrailingStopLossOrderRequestType] = {
    "FIXED_PRICE",
    "LIMIT",
    "MARKET",
    "MARKET_IF_TOUCHED",
    "STOP",
    "STOP_LOSS",
    "TAKE_PROFIT",
    "TRAILING_STOP_LOSS",
}


def check_trailing_stop_loss_order_request_type(
    value: str,
) -> TrailingStopLossOrderRequestType:
    if value in TRAILING_STOP_LOSS_ORDER_REQUEST_TYPE_VALUES:
        return cast(TrailingStopLossOrderRequestType, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRAILING_STOP_LOSS_ORDER_REQUEST_TYPE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .position import Position
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="ListPositionsResponse200")


@dataclasses.dataclass
class ListPositionsResponse200:
    """Attributes:
    positions (Optional[List['Position']]): The list of Account Positions.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    positions: Optional[List["Position"]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .position import Position

        d = src_dict.copy()
        positions = []
        _positions = d.pop("positions", None)
        for positions_item_data in _positions or []:
            positions_item = Position.from_dict(positions_item_data)
            positions.append(positions_item)
        last_transaction_id = d.pop("lastTransactionID", None)
        list_positions_response_200 = cls(
            positions=positions, last_transaction_id=last_transaction_id
        )
        list_positions_response_200.additional_properties = d
        return list_positions_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

AccountFinancingMode = Literal["DAILY", "NO_FINANCING", "SECOND_BY_SECOND"]
ACCOUNT_FINANCING_MODE_VALUES: Set[AccountFinancingMode] = {
    "DAILY",
    "NO_FINANCING",
    "SECOND_BY_SECOND",
}


def check_account_financing_mode(value: str) -> AccountFinancingMode:
    if value in ACCOUNT_FINANCING_MODE_VALUES:
        return cast(AccountFinancingMode, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {ACCOUNT_FINANCING_MODE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Type, TypeVar

T = TypeVar("T", bound="OrderRequest")


@dataclasses.dataclass
class OrderRequest:
    """The base Order specification used when requesting that an Order be created. Each specific Order-type extends this
    definition."""

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        order_request = cls()
        order_request.additional_properties = d
        return order_request

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.account_summary_guaranteed_stop_loss_order_mode import (
    AccountSummaryGuaranteedStopLossOrderMode,
)
from ..models.account_summary_guaranteed_stop_loss_order_mode import (
    check_account_summary_guaranteed_stop_loss_order_mode,
)
from typing import Union


T = TypeVar("T", bound="AccountSummary")


@_attrs_define
class AccountSummary:
    """A summary representation of a client's Account. The AccountSummary does not provide to full specification of pending
    Orders, open Trades and Positions.

        Attributes:
            id (Union[Unset, str]): The Account's identifier
            alias (Union[Unset, str]): Client-assigned alias for the Account. Only provided if the Account has an alias set
            currency (Union[Unset, str]): The home currency of the Account
            balance (Union[Unset, str]): The current balance of the Account.
            created_by_user_id (Union[Unset, int]): ID of the user that created the Account.
            created_time (Union[Unset, str]): The date/time when the Account was created.
            guaranteed_stop_loss_order_mode (Union[Unset, AccountSummaryGuaranteedStopLossOrderMode]): The current
                guaranteed Stop Loss Order mode of the Account.
            pl (Union[Unset, str]): The total profit/loss realized over the lifetime of the Account.
            resettable_pl (Union[Unset, str]): The total realized profit/loss for the Account since it was last reset by the
                client.
            resettable_pl_time (Union[Unset, str]): The date/time that the Account's resettablePL was last reset.
            financing (Union[Unset, str]): The total amount of financing paid/collected over the lifetime of the Account.
            commission (Union[Unset, str]): The total amount of commission paid over the lifetime of the Account.
            guaranteed_execution_fees (Union[Unset, str]): The total amount of fees charged over the lifetime of the Account
                for the execution of guaranteed Stop Loss Orders.
            margin_rate (Union[Unset, str]): Client-provided margin rate override for the Account. The effective margin rate
                of the Account is the lesser of this value and the OANDA margin rate for the Account's division. This value is
                only provided if a margin rate override exists for the Account.
            margin_call_enter_time (Union[Unset, str]): The date/time when the Account entered a margin call state. Only
                provided if the Account is in a margin call.
            margin_call_extension_count (Union[Unset, int]): The number of times that the Account's current margin call was
                extended.
            last_margin_call_extension_time (Union[Unset, str]): The date/time of the Account's last margin call extension.
            open_trade_count (Union[Unset, int]): The number of Trades currently open in the Account.
            open_position_count (Union[Unset, int]): The number of Positions currently open in the Account.
            pending_order_count (Union[Unset, int]): The number of Orders currently pending in the Account.
            hedging_enabled (Union[Unset, bool]): Flag indicating that the Account has hedging enabled.
            last_order_fill_timestamp (Union[Unset, str]): The date/time of the last order that was filled for this account.
            unrealized_pl (Union[Unset, str]): The total unrealized profit/loss for all Trades currently open in the
                Account.
            nav (Union[Unset, str]): The net asset value of the Account. Equal to Account balance + unrealizedPL.
            margin_used (Union[Unset, str]): Margin currently used for the Account.
            margin_available (Union[Unset, str]): Margin available for Account currency.
            position_value (Union[Unset, str]): The value of the Account's open positions represented in the Account's home
                currency.
            margin_closeout_unrealized_pl (Union[Unset, str]): The Account's margin closeout unrealized PL.
            margin_closeout_nav (Union[Unset, str]): The Account's margin closeout NAV.
            margin_closeout_margin_used (Union[Unset, str]): The Account's margin closeout margin used.
            margin_closeout_percent (Union[Unset, str]): The Account's margin closeout percentage. When this value is 1.0 or
                above the Account is in a margin closeout situation.
            margin_closeout_position_value (Union[Unset, str]): The value of the Account's open positions as used for margin
                closeout calculations represented in the Account's home currency.
            withdrawal_limit (Union[Unset, str]): The current WithdrawalLimit for the account which will be zero or a
                positive value indicating how much can be withdrawn from the account.
            margin_call_margin_used (Union[Unset, str]): The Account's margin call margin used.
            margin_call_percent (Union[Unset, str]): The Account's margin call percentage. When this value is 1.0 or above
                the Account is in a margin call situation.
            last_transaction_id (Union[Unset, str]): The ID of the last Transaction created for the Account.
    """

    id: Union[Unset, str] = UNSET
    alias: Union[Unset, str] = UNSET
    currency: Union[Unset, str] = UNSET
    balance: Union[Unset, str] = UNSET
    created_by_user_id: Union[Unset, int] = UNSET
    created_time: Union[Unset, str] = UNSET
    guaranteed_stop_loss_order_mode: Union[
        Unset, AccountSummaryGuaranteedStopLossOrderMode
    ] = UNSET
    pl: Union[Unset, str] = UNSET
    resettable_pl: Union[Unset, str] = UNSET
    resettable_pl_time: Union[Unset, str] = UNSET
    financing: Union[Unset, str] = UNSET
    commission: Union[Unset, str] = UNSET
    guaranteed_execution_fees: Union[Unset, str] = UNSET
    margin_rate: Union[Unset, str] = UNSET
    margin_call_enter_time: Union[Unset, str] = UNSET
    margin_call_extension_count: Union[Unset, int] = UNSET
    last_margin_call_extension_time: Union[Unset, str] = UNSET
    open_trade_count: Union[Unset, int] = UNSET
    open_position_count: Union[Unset, int] = UNSET
    pending_order_count: Union[Unset, int] = UNSET
    hedging_enabled: Union[Unset, bool] = UNSET
    last_order_fill_timestamp: Union[Unset, str] = UNSET
    unrealized_pl: Union[Unset, str] = UNSET
    nav: Union[Unset, str] = UNSET
    margin_used: Union[Unset, str] = UNSET
    margin_available: Union[Unset, str] = UNSET
    position_value: Union[Unset, str] = UNSET
    margin_closeout_unrealized_pl: Union[Unset, str] = UNSET
    margin_closeout_nav: Union[Unset, str] = UNSET
    margin_closeout_margin_used: Union[Unset, str] = UNSET
    margin_closeout_percent: Union[Unset, str] = UNSET
    margin_closeout_position_value: Union[Unset, str] = UNSET
    withdrawal_limit: Union[Unset, str] = UNSET
    margin_call_margin_used: Union[Unset, str] = UNSET
    margin_call_percent: Union[Unset, str] = UNSET
    last_transaction_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        alias = self.alias

        currency = self.currency

        balance = self.balance

        created_by_user_id = self.created_by_user_id

        created_time = self.created_time

        guaranteed_stop_loss_order_mode: Union[Unset, str] = UNSET
        if not isinstance(self.guaranteed_stop_loss_order_mode, Unset):
            guaranteed_stop_loss_order_mode = self.guaranteed_stop_loss_order_mode

        pl = self.pl

        resettable_pl = self.resettable_pl

        resettable_pl_time = self.resettable_pl_time

        financing = self.financing

        commission = self.commission

        guaranteed_execution_fees = self.guaranteed_execution_fees

        margin_rate = self.margin_rate

        margin_call_enter_time = self.margin_call_enter_time

        margin_call_extension_count = self.margin_call_extension_count

        last_margin_call_extension_time = self.last_margin_call_extension_time

        open_trade_count = self.open_trade_count

        open_position_count = self.open_position_count

        pending_order_count = self.pending_order_count

        hedging_enabled = self.hedging_enabled

        last_order_fill_timestamp = self.last_order_fill_timestamp

        unrealized_pl = self.unrealized_pl

        nav = self.nav

        margin_used = self.margin_used

        margin_available = self.margin_available

        position_value = self.position_value

        margin_closeout_unrealized_pl = self.margin_closeout_unrealized_pl

        margin_closeout_nav = self.margin_closeout_nav

        margin_closeout_margin_used = self.margin_closeout_margin_used

        margin_closeout_percent = self.margin_closeout_percent

        margin_closeout_position_value = self.margin_closeout_position_value

        withdrawal_limit = self.withdrawal_limit

        margin_call_margin_used = self.margin_call_margin_used

        margin_call_percent = self.margin_call_percent

        last_transaction_id = self.last_transaction_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if alias is not UNSET:
            field_dict["alias"] = alias
        if currency is not UNSET:
            field_dict["currency"] = currency
        if balance is not UNSET:
            field_dict["balance"] = balance
        if created_by_user_id is not UNSET:
            field_dict["createdByUserID"] = created_by_user_id
        if created_time is not UNSET:
            field_dict["createdTime"] = created_time
        if guaranteed_stop_loss_order_mode is not UNSET:
            field_dict["guaranteedStopLossOrderMode"] = guaranteed_stop_loss_order_mode
        if pl is not UNSET:
            field_dict["pl"] = pl
        if resettable_pl is not UNSET:
            field_dict["resettablePL"] = resettable_pl
        if resettable_pl_time is not UNSET:
            field_dict["resettablePLTime"] = resettable_pl_time
        if financing is not UNSET:
            field_dict["financing"] = financing
        if commission is not UNSET:
            field_dict["commission"] = commission
        if guaranteed_execution_fees is not UNSET:
            field_dict["guaranteedExecutionFees"] = guaranteed_execution_fees
        if margin_rate is not UNSET:
            field_dict["marginRate"] = margin_rate
        if margin_call_enter_time is not UNSET:
            field_dict["marginCallEnterTime"] = margin_call_enter_time
        if margin_call_extension_count is not UNSET:
            field_dict["marginCallExtensionCount"] = margin_call_extension_count
        if last_margin_call_extension_time is not UNSET:
            field_dict["lastMarginCallExtensionTime"] = last_margin_call_extension_time
        if open_trade_count is not UNSET:
            field_dict["openTradeCount"] = open_trade_count
        if open_position_count is not UNSET:
            field_dict["openPositionCount"] = open_position_count
        if pending_order_count is not UNSET:
            field_dict["pendingOrderCount"] = pending_order_count
        if hedging_enabled is not UNSET:
            field_dict["hedgingEnabled"] = hedging_enabled
        if last_order_fill_timestamp is not UNSET:
            field_dict["lastOrderFillTimestamp"] = last_order_fill_timestamp
        if unrealized_pl is not UNSET:
            field_dict["unrealizedPL"] = unrealized_pl
        if nav is not UNSET:
            field_dict["NAV"] = nav
        if margin_used is not UNSET:
            field_dict["marginUsed"] = margin_used
        if margin_available is not UNSET:
            field_dict["marginAvailable"] = margin_available
        if position_value is not UNSET:
            field_dict["positionValue"] = position_value
        if margin_closeout_unrealized_pl is not UNSET:
            field_dict["marginCloseoutUnrealizedPL"] = margin_closeout_unrealized_pl
        if margin_closeout_nav is not UNSET:
            field_dict["marginCloseoutNAV"] = margin_closeout_nav
        if margin_closeout_margin_used is not UNSET:
            field_dict["marginCloseoutMarginUsed"] = margin_closeout_margin_used
        if margin_closeout_percent is not UNSET:
            field_dict["marginCloseoutPercent"] = margin_closeout_percent
        if margin_closeout_position_value is not UNSET:
            field_dict["marginCloseoutPositionValue"] = margin_closeout_position_value
        if withdrawal_limit is not UNSET:
            field_dict["withdrawalLimit"] = withdrawal_limit
        if margin_call_margin_used is not UNSET:
            field_dict["marginCallMarginUsed"] = margin_call_margin_used
        if margin_call_percent is not UNSET:
            field_dict["marginCallPercent"] = margin_call_percent
        if last_transaction_id is not UNSET:
            field_dict["lastTransactionID"] = last_transaction_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        alias = d.pop("alias", UNSET)

        currency = d.pop("currency", UNSET)

        balance = d.pop("balance", UNSET)

        created_by_user_id = d.pop("createdByUserID", UNSET)

        created_time = d.pop("createdTime", UNSET)

        _guaranteed_stop_loss_order_mode = d.pop("guaranteedStopLossOrderMode", UNSET)
        guaranteed_stop_loss_order_mode: Union[
            Unset, AccountSummaryGuaranteedStopLossOrderMode
        ]
        if isinstance(_guaranteed_stop_loss_order_mode, Unset):
            guaranteed_stop_loss_order_mode = UNSET
        else:
            guaranteed_stop_loss_order_mode = (
                check_account_summary_guaranteed_stop_loss_order_mode(
                    _guaranteed_stop_loss_order_mode
                )
            )

        pl = d.pop("pl", UNSET)

        resettable_pl = d.pop("resettablePL", UNSET)

        resettable_pl_time = d.pop("resettablePLTime", UNSET)

        financing = d.pop("financing", UNSET)

        commission = d.pop("commission", UNSET)

        guaranteed_execution_fees = d.pop("guaranteedExecutionFees", UNSET)

        margin_rate = d.pop("marginRate", UNSET)

        margin_call_enter_time = d.pop("marginCallEnterTime", UNSET)

        margin_call_extension_count = d.pop("marginCallExtensionCount", UNSET)

        last_margin_call_extension_time = d.pop("lastMarginCallExtensionTime", UNSET)

        open_trade_count = d.pop("openTradeCount", UNSET)

        open_position_count = d.pop("openPositionCount", UNSET)

        pending_order_count = d.pop("pendingOrderCount", UNSET)

        hedging_enabled = d.pop("hedgingEnabled", UNSET)

        last_order_fill_timestamp = d.pop("lastOrderFillTimestamp", UNSET)

        unrealized_pl = d.pop("unrealizedPL", UNSET)

        nav = d.pop("NAV", UNSET)

        margin_used = d.pop("marginUsed", UNSET)

        margin_available = d.pop("marginAvailable", UNSET)

        position_value = d.pop("positionValue", UNSET)

        margin_closeout_unrealized_pl = d.pop("marginCloseoutUnrealizedPL", UNSET)

        margin_closeout_nav = d.pop("marginCloseoutNAV", UNSET)

        margin_closeout_margin_used = d.pop("marginCloseoutMarginUsed", UNSET)

        margin_closeout_percent = d.pop("marginCloseoutPercent", UNSET)

        margin_closeout_position_value = d.pop("marginCloseoutPositionValue", UNSET)

        withdrawal_limit = d.pop("withdrawalLimit", UNSET)

        margin_call_margin_used = d.pop("marginCallMarginUsed", UNSET)

        margin_call_percent = d.pop("marginCallPercent", UNSET)

        last_transaction_id = d.pop("lastTransactionID", UNSET)

        account_summary = cls(
            id=id,
            alias=alias,
            currency=currency,
            balance=balance,
            created_by_user_id=created_by_user_id,
            created_time=created_time,
            guaranteed_stop_loss_order_mode=guaranteed_stop_loss_order_mode,
            pl=pl,
            resettable_pl=resettable_pl,
            resettable_pl_time=resettable_pl_time,
            financing=financing,
            commission=commission,
            guaranteed_execution_fees=guaranteed_execution_fees,
            margin_rate=margin_rate,
            margin_call_enter_time=margin_call_enter_time,
            margin_call_extension_count=margin_call_extension_count,
            last_margin_call_extension_time=last_margin_call_extension_time,
            open_trade_count=open_trade_count,
            open_position_count=open_position_count,
            pending_order_count=pending_order_count,
            hedging_enabled=hedging_enabled,
            last_order_fill_timestamp=last_order_fill_timestamp,
            unrealized_pl=unrealized_pl,
            nav=nav,
            margin_used=margin_used,
            margin_available=margin_available,
            position_value=position_value,
            margin_closeout_unrealized_pl=margin_closeout_unrealized_pl,
            margin_closeout_nav=margin_closeout_nav,
            margin_closeout_margin_used=margin_closeout_margin_used,
            margin_closeout_percent=margin_closeout_percent,
            margin_closeout_position_value=margin_closeout_position_value,
            withdrawal_limit=withdrawal_limit,
            margin_call_margin_used=margin_call_margin_used,
            margin_call_percent=margin_call_percent,
            last_transaction_id=last_transaction_id,
        )

        account_summary.additional_properties = d
        return account_summary

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

DailyFinancingTransactionAccountFinancingMode = Literal[
    "DAILY", "NO_FINANCING", "SECOND_BY_SECOND"
]
DAILY_FINANCING_TRANSACTION_ACCOUNT_FINANCING_MODE_VALUES: Set[
    DailyFinancingTransactionAccountFinancingMode
] = {"DAILY", "NO_FINANCING", "SECOND_BY_SECOND"}


def check_daily_financing_transaction_account_financing_mode(
    value: str,
) -> DailyFinancingTransactionAccountFinancingMode:
    if value in DAILY_FINANCING_TRANSACTION_ACCOUNT_FINANCING_MODE_VALUES:
        return cast(DailyFinancingTransactionAccountFinancingMode, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {DAILY_FINANCING_TRANSACTION_ACCOUNT_FINANCING_MODE_VALUES!r}"
    )



from __future__ import annotations
from typing import Literal, Set, cast

StopOrderRequestTriggerCondition = Literal["ASK", "BID", "DEFAULT", "INVERSE", "MID"]
STOP_ORDER_REQUEST_TRIGGER_CONDITION_VALUES: Set[StopOrderRequestTriggerCondition] = {
    "ASK",
    "BID",
    "DEFAULT",
    "INVERSE",
    "MID",
}


def check_stop_order_request_trigger_condition(
    value: str,
) -> StopOrderRequestTriggerCondition:
    if value in STOP_ORDER_REQUEST_TRIGGER_CONDITION_VALUES:
        return cast(StopOrderRequestTriggerCondition, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_ORDER_REQUEST_TRIGGER_CONDITION_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="GuaranteedStopLossOrderLevelRestriction")


@_attrs_define
class GuaranteedStopLossOrderLevelRestriction:
    """A GuaranteedStopLossOrderLevelRestriction represents the total position size that can exist within a given price
    window for Trades with guaranteed Stop Loss Orders attached for a specific Instrument.

        Attributes:
            volume (Union[Unset, str]): Applies to Trades with a guaranteed Stop Loss Order attached for the specified
                Instrument. This is the total allowed Trade volume that can exist within the priceRange based on the trigger
                prices of the guaranteed Stop Loss Orders.
            price_range (Union[Unset, str]): The price range the volume applies to. This value is in price units.
    """

    volume: Union[Unset, str] = UNSET
    price_range: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        volume = self.volume

        price_range = self.price_range

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if volume is not UNSET:
            field_dict["volume"] = volume
        if price_range is not UNSET:
            field_dict["priceRange"] = price_range

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        volume = d.pop("volume", UNSET)

        price_range = d.pop("priceRange", UNSET)

        guaranteed_stop_loss_order_level_restriction = cls(
            volume=volume,
            price_range=price_range,
        )

        guaranteed_stop_loss_order_level_restriction.additional_properties = d
        return guaranteed_stop_loss_order_level_restriction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .limit_order_request_position_fill import LimitOrderRequestPositionFill
from .limit_order_request_position_fill import check_limit_order_request_position_fill
from .limit_order_request_time_in_force import LimitOrderRequestTimeInForce
from .limit_order_request_time_in_force import check_limit_order_request_time_in_force
from .limit_order_request_trigger_condition import LimitOrderRequestTriggerCondition
from .limit_order_request_trigger_condition import (
    check_limit_order_request_trigger_condition,
)
from .limit_order_request_type import LimitOrderRequestType
from .limit_order_request_type import check_limit_order_request_type
from .stop_loss_details import StopLossDetails
from .take_profit_details import TakeProfitDetails
from .trailing_stop_loss_details import TrailingStopLossDetails
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="LimitOrderRequest")


@dataclasses.dataclass
class LimitOrderRequest:
    """A LimitOrderRequest specifies the parameters that may be set when creating a Limit Order.

    Attributes:
        type (Optional[LimitOrderRequestType]): The type of the Order to Create. Must be set to "LIMIT" when
            creating a Market Order.
        instrument (Optional[str]): The Limit Order's Instrument.
        units (Optional[str]): The quantity requested to be filled by the Limit Order. A posititive number of units
            results in a long Order, and a negative number of units results in a short Order.
        price (Optional[str]): The price threshold specified for the Limit Order. The Limit Order will only be
            filled by a market price that is equal to or better than this price.
        time_in_force (Optional[LimitOrderRequestTimeInForce]): The time-in-force requested for the Limit Order.
        gtd_time (Optional[str]): The date/time when the Limit Order will be cancelled if its timeInForce is "GTD".
        position_fill (Optional[LimitOrderRequestPositionFill]): Specification of how Positions in the Account are
            modified when the Order is filled.
        trigger_condition (Optional[LimitOrderRequestTriggerCondition]): Specification of which price component
            should be used when determining if an Order should be triggered and filled. This allows Orders to be triggered
            based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy) price
            depending on the desired behaviour. Orders are always filled using their default price component.
            This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
            condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
            transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
            is set to the default value when indicating the distance from an Order's trigger price, and will always provide
            the default trigger condition when creating or modifying an Order.
            A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
            value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
            a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
        client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        take_profit_on_fill (Optional[TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
            Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
            a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss_on_fill (Optional[StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
            Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss_on_fill (Optional[TrailingStopLossDetails]): TrailingStopLossDetails specifies the
            details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
            filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
            is modified directly through the Trade.
        trade_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4."""

    type: Optional[LimitOrderRequestType]
    instrument: Optional[str]
    units: Optional[str]
    price: Optional[str]
    time_in_force: Optional[LimitOrderRequestTimeInForce]
    gtd_time: Optional[str]
    position_fill: Optional[LimitOrderRequestPositionFill]
    trigger_condition: Optional[LimitOrderRequestTriggerCondition]
    client_extensions: Optional["ClientExtensions"]
    take_profit_on_fill: Optional["TakeProfitDetails"]
    stop_loss_on_fill: Optional["StopLossDetails"]
    trailing_stop_loss_on_fill: Optional["TrailingStopLossDetails"]
    trade_client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .take_profit_details import TakeProfitDetails
        from .trailing_stop_loss_details import TrailingStopLossDetails
        from .client_extensions import ClientExtensions
        from .stop_loss_details import StopLossDetails

        d = src_dict.copy()
        _type = d.pop("type", None)
        type: Optional[LimitOrderRequestType]
        if _type is None:
            type = None
        else:
            type = check_limit_order_request_type(_type)
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        price = d.pop("price", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[LimitOrderRequestTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_limit_order_request_time_in_force(_time_in_force)
        gtd_time = d.pop("gtdTime", None)
        _position_fill = d.pop("positionFill", None)
        position_fill: Optional[LimitOrderRequestPositionFill]
        if _position_fill is None:
            position_fill = None
        else:
            position_fill = check_limit_order_request_position_fill(_position_fill)
        _trigger_condition = d.pop("triggerCondition", None)
        trigger_condition: Optional[LimitOrderRequestTriggerCondition]
        if _trigger_condition is None:
            trigger_condition = None
        else:
            trigger_condition = check_limit_order_request_trigger_condition(
                _trigger_condition
            )
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _take_profit_on_fill = d.pop("takeProfitOnFill", None)
        take_profit_on_fill: Optional[TakeProfitDetails]
        if _take_profit_on_fill is None:
            take_profit_on_fill = None
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)
        _stop_loss_on_fill = d.pop("stopLossOnFill", None)
        stop_loss_on_fill: Optional[StopLossDetails]
        if _stop_loss_on_fill is None:
            stop_loss_on_fill = None
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)
        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", None)
        trailing_stop_loss_on_fill: Optional[TrailingStopLossDetails]
        if _trailing_stop_loss_on_fill is None:
            trailing_stop_loss_on_fill = None
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )
        _trade_client_extensions = d.pop("tradeClientExtensions", None)
        trade_client_extensions: Optional[ClientExtensions]
        if _trade_client_extensions is None:
            trade_client_extensions = None
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )
        limit_order_request = cls(
            type=type,
            instrument=instrument,
            units=units,
            price=price,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            position_fill=position_fill,
            trigger_condition=trigger_condition,
            client_extensions=client_extensions,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
        )
        limit_order_request.additional_properties = d
        return limit_order_request

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order import Order
from .position import Position
from .trade_summary import TradeSummary
from .transaction import Transaction
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="AccountChanges")


@dataclasses.dataclass
class AccountChanges:
    """An AccountChanges Object is used to represent the changes to an Account's Orders, Trades and Positions since a
    specified Account TransactionID in the past.

        Attributes:
            orders_created (Optional[List['Order']]): The Orders created. These Orders may have been filled, cancelled
                or triggered in the same period.
            orders_cancelled (Optional[List['Order']]): The Orders cancelled.
            orders_filled (Optional[List['Order']]): The Orders filled.
            orders_triggered (Optional[List['Order']]): The Orders triggered.
            trades_opened (Optional[List['TradeSummary']]): The Trades opened.
            trades_reduced (Optional[List['TradeSummary']]): The Trades reduced.
            trades_closed (Optional[List['TradeSummary']]): The Trades closed.
            positions (Optional[List['Position']]): The Positions changed.
            transactions (Optional[List['Transaction']]): The Transactions that have been generated."""

    orders_created: Optional[List["Order"]]
    orders_cancelled: Optional[List["Order"]]
    orders_filled: Optional[List["Order"]]
    orders_triggered: Optional[List["Order"]]
    trades_opened: Optional[List["TradeSummary"]]
    trades_reduced: Optional[List["TradeSummary"]]
    trades_closed: Optional[List["TradeSummary"]]
    positions: Optional[List["Position"]]
    transactions: Optional[List["Transaction"]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .transaction import Transaction
        from .order import Order
        from .trade_summary import TradeSummary
        from .position import Position

        d = src_dict.copy()
        orders_created = []
        _orders_created = d.pop("ordersCreated", None)
        for orders_created_item_data in _orders_created or []:
            orders_created_item = Order.from_dict(orders_created_item_data)
            orders_created.append(orders_created_item)
        orders_cancelled = []
        _orders_cancelled = d.pop("ordersCancelled", None)
        for orders_cancelled_item_data in _orders_cancelled or []:
            orders_cancelled_item = Order.from_dict(orders_cancelled_item_data)
            orders_cancelled.append(orders_cancelled_item)
        orders_filled = []
        _orders_filled = d.pop("ordersFilled", None)
        for orders_filled_item_data in _orders_filled or []:
            orders_filled_item = Order.from_dict(orders_filled_item_data)
            orders_filled.append(orders_filled_item)
        orders_triggered = []
        _orders_triggered = d.pop("ordersTriggered", None)
        for orders_triggered_item_data in _orders_triggered or []:
            orders_triggered_item = Order.from_dict(orders_triggered_item_data)
            orders_triggered.append(orders_triggered_item)
        trades_opened = []
        _trades_opened = d.pop("tradesOpened", None)
        for trades_opened_item_data in _trades_opened or []:
            trades_opened_item = TradeSummary.from_dict(trades_opened_item_data)
            trades_opened.append(trades_opened_item)
        trades_reduced = []
        _trades_reduced = d.pop("tradesReduced", None)
        for trades_reduced_item_data in _trades_reduced or []:
            trades_reduced_item = TradeSummary.from_dict(trades_reduced_item_data)
            trades_reduced.append(trades_reduced_item)
        trades_closed = []
        _trades_closed = d.pop("tradesClosed", None)
        for trades_closed_item_data in _trades_closed or []:
            trades_closed_item = TradeSummary.from_dict(trades_closed_item_data)
            trades_closed.append(trades_closed_item)
        positions = []
        _positions = d.pop("positions", None)
        for positions_item_data in _positions or []:
            positions_item = Position.from_dict(positions_item_data)
            positions.append(positions_item)
        transactions = []
        _transactions = d.pop("transactions", None)
        for transactions_item_data in _transactions or []:
            transactions_item = Transaction.from_dict(transactions_item_data)
            transactions.append(transactions_item)
        account_changes = cls(
            orders_created=orders_created,
            orders_cancelled=orders_cancelled,
            orders_filled=orders_filled,
            orders_triggered=orders_triggered,
            trades_opened=trades_opened,
            trades_reduced=trades_reduced,
            trades_closed=trades_closed,
            positions=positions,
            transactions=transactions,
        )
        account_changes.additional_properties = d
        return account_changes

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.home_conversions import HomeConversions
    from ..models.client_price import ClientPrice


T = TypeVar("T", bound="GetPricesResponse200")


@_attrs_define
class GetPricesResponse200:
    """
    Attributes:
        prices (Union[Unset, List['ClientPrice']]): The list of Price objects requested.
        home_conversions (Union[Unset, List['HomeConversions']]): The list of home currency conversion factors
            requested. This field will only be present if includeHomeConversions was set to true in the request.
        time (Union[Unset, str]): The DateTime value to use for the "since" parameter in the next poll request.
    """

    prices: Union[Unset, List["ClientPrice"]] = UNSET
    home_conversions: Union[Unset, List["HomeConversions"]] = UNSET
    time: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        prices: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.prices, Unset):
            prices = []
            for prices_item_data in self.prices:
                prices_item = prices_item_data.to_dict()
                prices.append(prices_item)

        home_conversions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.home_conversions, Unset):
            home_conversions = []
            for home_conversions_item_data in self.home_conversions:
                home_conversions_item = home_conversions_item_data.to_dict()
                home_conversions.append(home_conversions_item)

        time = self.time

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if prices is not UNSET:
            field_dict["prices"] = prices
        if home_conversions is not UNSET:
            field_dict["homeConversions"] = home_conversions
        if time is not UNSET:
            field_dict["time"] = time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.home_conversions import HomeConversions
        from ..models.client_price import ClientPrice

        d = src_dict.copy()
        prices = []
        _prices = d.pop("prices", UNSET)
        for prices_item_data in _prices or []:
            prices_item = ClientPrice.from_dict(prices_item_data)

            prices.append(prices_item)

        home_conversions = []
        _home_conversions = d.pop("homeConversions", UNSET)
        for home_conversions_item_data in _home_conversions or []:
            home_conversions_item = HomeConversions.from_dict(
                home_conversions_item_data
            )

            home_conversions.append(home_conversions_item)

        time = d.pop("time", UNSET)

        get_prices_response_200 = cls(
            prices=prices,
            home_conversions=home_conversions,
            time=time,
        )

        get_prices_response_200.additional_properties = d
        return get_prices_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

TakeProfitOrderReason = Literal["CLIENT_ORDER", "ON_FILL", "REPLACEMENT"]

TAKE_PROFIT_ORDER_REASON_VALUES: Set[TakeProfitOrderReason] = {
    "CLIENT_ORDER",
    "ON_FILL",
    "REPLACEMENT",
}


def check_take_profit_order_reason(value: str) -> TakeProfitOrderReason:
    if value in TAKE_PROFIT_ORDER_REASON_VALUES:
        return cast(TakeProfitOrderReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TAKE_PROFIT_ORDER_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .trade_client_extensions_modify_reject_transaction import (
    TradeClientExtensionsModifyRejectTransaction,
)
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="SetTradeClientExtensionsResponse400")


@dataclasses.dataclass
class SetTradeClientExtensionsResponse400:
    """Attributes:
    trade_client_extensions_modify_reject_transaction (Optional[TradeClientExtensionsModifyRejectTransaction]):
        A TradeClientExtensionsModifyRejectTransaction represents the rejection of the modification of a Trade's Client
        Extensions.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account.
    related_transaction_i_ds (Optional[List[str]]): The IDs of all Transactions that were created while
        satisfying the request.
    error_code (Optional[str]): The code of the error that has occurred. This field may not be returned for some
        errors.
    error_message (Optional[str]): The human-readable description of the error that has occurred."""

    trade_client_extensions_modify_reject_transaction: Optional[
        "TradeClientExtensionsModifyRejectTransaction"
    ]
    last_transaction_id: Optional[str]
    related_transaction_i_ds: Optional[List[str]]
    error_code: Optional[str]
    error_message: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .trade_client_extensions_modify_reject_transaction import (
            TradeClientExtensionsModifyRejectTransaction,
        )

        d = src_dict.copy()
        _trade_client_extensions_modify_reject_transaction = d.pop(
            "tradeClientExtensionsModifyRejectTransaction", None
        )
        trade_client_extensions_modify_reject_transaction: Optional[
            TradeClientExtensionsModifyRejectTransaction
        ]
        if _trade_client_extensions_modify_reject_transaction is None:
            trade_client_extensions_modify_reject_transaction = None
        else:
            trade_client_extensions_modify_reject_transaction = (
                TradeClientExtensionsModifyRejectTransaction.from_dict(
                    _trade_client_extensions_modify_reject_transaction
                )
            )
        last_transaction_id = d.pop("lastTransactionID", None)
        related_transaction_i_ds = cast(List[str], d.pop("relatedTransactionIDs", None))
        error_code = d.pop("errorCode", None)
        error_message = d.pop("errorMessage", None)
        set_trade_client_extensions_response_400 = cls(
            trade_client_extensions_modify_reject_transaction=trade_client_extensions_modify_reject_transaction,
            last_transaction_id=last_transaction_id,
            related_transaction_i_ds=related_transaction_i_ds,
            error_code=error_code,
            error_message=error_message,
        )
        set_trade_client_extensions_response_400.additional_properties = d
        return set_trade_client_extensions_response_400

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

StopOrderTransactionTriggerCondition = Literal[
    "ASK", "BID", "DEFAULT", "INVERSE", "MID"
]
STOP_ORDER_TRANSACTION_TRIGGER_CONDITION_VALUES: Set[
    StopOrderTransactionTriggerCondition
] = {"ASK", "BID", "DEFAULT", "INVERSE", "MID"}


def check_stop_order_transaction_trigger_condition(
    value: str,
) -> StopOrderTransactionTriggerCondition:
    if value in STOP_ORDER_TRANSACTION_TRIGGER_CONDITION_VALUES:
        return cast(StopOrderTransactionTriggerCondition, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_ORDER_TRANSACTION_TRIGGER_CONDITION_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .instrument import Instrument
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="GetAccountInstrumentsResponse200")


@dataclasses.dataclass
class GetAccountInstrumentsResponse200:
    """Attributes:
    instruments (Optional[List['Instrument']]): The requested list of instruments.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account."""

    instruments: Optional[List["Instrument"]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .instrument import Instrument

        d = src_dict.copy()
        instruments = []
        _instruments = d.pop("instruments", None)
        for instruments_item_data in _instruments or []:
            instruments_item = Instrument.from_dict(instruments_item_data)
            instruments.append(instruments_item)
        last_transaction_id = d.pop("lastTransactionID", None)
        get_account_instruments_response_200 = cls(
            instruments=instruments, last_transaction_id=last_transaction_id
        )
        get_account_instruments_response_200.additional_properties = d
        return get_account_instruments_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

LimitOrderTransactionPositionFill = Literal[
    "DEFAULT", "OPEN_ONLY", "REDUCE_FIRST", "REDUCE_ONLY"
]
LIMIT_ORDER_TRANSACTION_POSITION_FILL_VALUES: Set[LimitOrderTransactionPositionFill] = {
    "DEFAULT",
    "OPEN_ONLY",
    "REDUCE_FIRST",
    "REDUCE_ONLY",
}


def check_limit_order_transaction_position_fill(
    value: str,
) -> LimitOrderTransactionPositionFill:
    if value in LIMIT_ORDER_TRANSACTION_POSITION_FILL_VALUES:
        return cast(LimitOrderTransactionPositionFill, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {LIMIT_ORDER_TRANSACTION_POSITION_FILL_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .guaranteed_stop_loss_order_level_restriction import (
    GuaranteedStopLossOrderLevelRestriction,
)
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GuaranteedStopLossOrderEntryData")


@dataclasses.dataclass
class GuaranteedStopLossOrderEntryData:
    """Details required by clients creating a Guaranteed Stop Loss Order

    Attributes:
        minimum_distance (Optional[str]): The minimum distance allowed between the Trade's fill price and the
            configured price for guaranteed Stop Loss Orders created for this instrument. Specified in price units.
        premium (Optional[str]): The amount that is charged to the account if a guaranteed Stop Loss Order is
            triggered and filled. The value is in price units and is charged for each unit of the Trade.
        level_restriction (Optional[GuaranteedStopLossOrderLevelRestriction]): A
            GuaranteedStopLossOrderLevelRestriction represents the total position size that can exist within a given price
            window for Trades with guaranteed Stop Loss Orders attached for a specific Instrument."""

    minimum_distance: Optional[str]
    premium: Optional[str]
    level_restriction: Optional["GuaranteedStopLossOrderLevelRestriction"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .guaranteed_stop_loss_order_level_restriction import (
            GuaranteedStopLossOrderLevelRestriction,
        )

        d = src_dict.copy()
        minimum_distance = d.pop("minimumDistance", None)
        premium = d.pop("premium", None)
        _level_restriction = d.pop("levelRestriction", None)
        level_restriction: Optional[GuaranteedStopLossOrderLevelRestriction]
        if _level_restriction is None:
            level_restriction = None
        else:
            level_restriction = GuaranteedStopLossOrderLevelRestriction.from_dict(
                _level_restriction
            )
        guaranteed_stop_loss_order_entry_data = cls(
            minimum_distance=minimum_distance,
            premium=premium,
            level_restriction=level_restriction,
        )
        guaranteed_stop_loss_order_entry_data.additional_properties = d
        return guaranteed_stop_loss_order_entry_data

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="LiquidityRegenerationScheduleStep")


@dataclasses.dataclass
class LiquidityRegenerationScheduleStep:
    """A liquidity regeneration schedule Step indicates the amount of bid and ask liquidity that is used by the Account at
    a certain time. These amounts will only change at the timestamp of the following step.

        Attributes:
            timestamp (Optional[str]): The timestamp of the schedule step.
            bid_liquidity_used (Optional[str]): The amount of bid liquidity used at this step in the schedule.
            ask_liquidity_used (Optional[str]): The amount of ask liquidity used at this step in the schedule."""

    timestamp: Optional[str]
    bid_liquidity_used: Optional[str]
    ask_liquidity_used: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        timestamp = d.pop("timestamp", None)
        bid_liquidity_used = d.pop("bidLiquidityUsed", None)
        ask_liquidity_used = d.pop("askLiquidityUsed", None)
        liquidity_regeneration_schedule_step = cls(
            timestamp=timestamp,
            bid_liquidity_used=bid_liquidity_used,
            ask_liquidity_used=ask_liquidity_used,
        )
        liquidity_regeneration_schedule_step.additional_properties = d
        return liquidity_regeneration_schedule_step

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order_cancel_transaction import OrderCancelTransaction
from .order_fill_transaction import OrderFillTransaction
from .transaction import Transaction
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="ReplaceOrderResponse201")


@dataclasses.dataclass
class ReplaceOrderResponse201:
    """Attributes:
    order_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction represents the
        cancellation of an Order in the client's Account.
    order_create_transaction (Optional[Transaction]): The base Transaction specification. Specifies properties
        that are common between all Transaction.
    order_fill_transaction (Optional[OrderFillTransaction]): An OrderFillTransaction represents the filling of
        an Order in the client's Account.
    order_reissue_transaction (Optional[Transaction]): The base Transaction specification. Specifies properties
        that are common between all Transaction.
    order_reissue_reject_transaction (Optional[Transaction]): The base Transaction specification. Specifies
        properties that are common between all Transaction.
    replacing_order_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction represents
        the cancellation of an Order in the client's Account.
    related_transaction_i_ds (Optional[List[str]]): The IDs of all Transactions that were created while
        satisfying the request.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    order_cancel_transaction: Optional["OrderCancelTransaction"]
    order_create_transaction: Optional["Transaction"]
    order_fill_transaction: Optional["OrderFillTransaction"]
    order_reissue_transaction: Optional["Transaction"]
    order_reissue_reject_transaction: Optional["Transaction"]
    replacing_order_cancel_transaction: Optional["OrderCancelTransaction"]
    related_transaction_i_ds: Optional[List[str]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .transaction import Transaction
        from .order_fill_transaction import OrderFillTransaction
        from .order_cancel_transaction import OrderCancelTransaction

        d = src_dict.copy()
        _order_cancel_transaction = d.pop("orderCancelTransaction", None)
        order_cancel_transaction: Optional[OrderCancelTransaction]
        if _order_cancel_transaction is None:
            order_cancel_transaction = None
        else:
            order_cancel_transaction = OrderCancelTransaction.from_dict(
                _order_cancel_transaction
            )
        _order_create_transaction = d.pop("orderCreateTransaction", None)
        order_create_transaction: Optional[Transaction]
        if _order_create_transaction is None:
            order_create_transaction = None
        else:
            order_create_transaction = Transaction.from_dict(_order_create_transaction)
        _order_fill_transaction = d.pop("orderFillTransaction", None)
        order_fill_transaction: Optional[OrderFillTransaction]
        if _order_fill_transaction is None:
            order_fill_transaction = None
        else:
            order_fill_transaction = OrderFillTransaction.from_dict(
                _order_fill_transaction
            )
        _order_reissue_transaction = d.pop("orderReissueTransaction", None)
        order_reissue_transaction: Optional[Transaction]
        if _order_reissue_transaction is None:
            order_reissue_transaction = None
        else:
            order_reissue_transaction = Transaction.from_dict(
                _order_reissue_transaction
            )
        _order_reissue_reject_transaction = d.pop("orderReissueRejectTransaction", None)
        order_reissue_reject_transaction: Optional[Transaction]
        if _order_reissue_reject_transaction is None:
            order_reissue_reject_transaction = None
        else:
            order_reissue_reject_transaction = Transaction.from_dict(
                _order_reissue_reject_transaction
            )
        _replacing_order_cancel_transaction = d.pop(
            "replacingOrderCancelTransaction", None
        )
        replacing_order_cancel_transaction: Optional[OrderCancelTransaction]
        if _replacing_order_cancel_transaction is None:
            replacing_order_cancel_transaction = None
        else:
            replacing_order_cancel_transaction = OrderCancelTransaction.from_dict(
                _replacing_order_cancel_transaction
            )
        related_transaction_i_ds = cast(List[str], d.pop("relatedTransactionIDs", None))
        last_transaction_id = d.pop("lastTransactionID", None)
        replace_order_response_201 = cls(
            order_cancel_transaction=order_cancel_transaction,
            order_create_transaction=order_create_transaction,
            order_fill_transaction=order_fill_transaction,
            order_reissue_transaction=order_reissue_transaction,
            order_reissue_reject_transaction=order_reissue_reject_transaction,
            replacing_order_cancel_transaction=replacing_order_cancel_transaction,
            related_transaction_i_ds=related_transaction_i_ds,
            last_transaction_id=last_transaction_id,
        )
        replace_order_response_201.additional_properties = d
        return replace_order_response_201

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order_book_bucket import OrderBookBucket
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="OrderBook")


@dataclasses.dataclass
class OrderBook:
    """The representation of an instrument's order book at a point in time

    Attributes:
        instrument (Optional[str]): The order book's instrument
        time (Optional[str]): The time when the order book snapshot was created.
        price (Optional[str]): The price (midpoint) for the order book's instrument at the time of the order book
            snapshot
        bucket_width (Optional[str]): The price width for each bucket. Each bucket covers the price range from the
            bucket's price to the bucket's price + bucketWidth.
        buckets (Optional[List['OrderBookBucket']]): The partitioned order book, divided into buckets using a
            default bucket width. These buckets are only provided for price ranges which actually contain order or position
            data."""

    instrument: Optional[str]
    time: Optional[str]
    price: Optional[str]
    bucket_width: Optional[str]
    buckets: Optional[List["OrderBookBucket"]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .order_book_bucket import OrderBookBucket

        d = src_dict.copy()
        instrument = d.pop("instrument", None)
        time = d.pop("time", None)
        price = d.pop("price", None)
        bucket_width = d.pop("bucketWidth", None)
        buckets = []
        _buckets = d.pop("buckets", None)
        for buckets_item_data in _buckets or []:
            buckets_item = OrderBookBucket.from_dict(buckets_item_data)
            buckets.append(buckets_item)
        order_book = cls(
            instrument=instrument,
            time=time,
            price=price,
            bucket_width=bucket_width,
            buckets=buckets,
        )
        order_book.additional_properties = d
        return order_book

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_configure_reject_transaction_reject_reason import (
    ClientConfigureRejectTransactionRejectReason,
)
from .client_configure_reject_transaction_reject_reason import (
    check_client_configure_reject_transaction_reject_reason,
)
from .client_configure_reject_transaction_type import (
    ClientConfigureRejectTransactionType,
)
from .client_configure_reject_transaction_type import (
    check_client_configure_reject_transaction_type,
)
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="ClientConfigureRejectTransaction")


@dataclasses.dataclass
class ClientConfigureRejectTransaction:
    """A ClientConfigureRejectTransaction represents the reject of configuration of an Account by a client.

    Attributes:
        id (Optional[str]): The Transaction's Identifier.
        time (Optional[str]): The date/time when the Transaction was created.
        user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Optional[str]): The ID of the Account the Transaction was created for.
        batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Optional[str]): The Request ID of the request which generated the transaction.
        type (Optional[ClientConfigureRejectTransactionType]): The Type of the Transaction. Always set to
            "CLIENT_CONFIGURE_REJECT" in a ClientConfigureRejectTransaction.
        alias (Optional[str]): The client-provided alias for the Account.
        margin_rate (Optional[str]): The margin rate override for the Account.
        reject_reason (Optional[ClientConfigureRejectTransactionRejectReason]): The reason that the Reject
            Transaction was created"""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[ClientConfigureRejectTransactionType]
    alias: Optional[str]
    margin_rate: Optional[str]
    reject_reason: Optional[ClientConfigureRejectTransactionRejectReason]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[ClientConfigureRejectTransactionType]
        if _type is None:
            type = None
        else:
            type = check_client_configure_reject_transaction_type(_type)
        alias = d.pop("alias", None)
        margin_rate = d.pop("marginRate", None)
        _reject_reason = d.pop("rejectReason", None)
        reject_reason: Optional[ClientConfigureRejectTransactionRejectReason]
        if _reject_reason is None:
            reject_reason = None
        else:
            reject_reason = check_client_configure_reject_transaction_reject_reason(
                _reject_reason
            )
        client_configure_reject_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            alias=alias,
            margin_rate=margin_rate,
            reject_reason=reject_reason,
        )
        client_configure_reject_transaction.additional_properties = d
        return client_configure_reject_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.limit_order_position_fill import check_limit_order_position_fill
from ..models.limit_order_position_fill import LimitOrderPositionFill
from ..models.limit_order_state import check_limit_order_state
from ..models.limit_order_state import LimitOrderState
from ..models.limit_order_time_in_force import check_limit_order_time_in_force
from ..models.limit_order_time_in_force import LimitOrderTimeInForce
from ..models.limit_order_trigger_condition import check_limit_order_trigger_condition
from ..models.limit_order_trigger_condition import LimitOrderTriggerCondition
from ..models.limit_order_type import check_limit_order_type
from ..models.limit_order_type import LimitOrderType
from typing import cast
from typing import Union

if TYPE_CHECKING:
    from ..models.take_profit_details import TakeProfitDetails
    from ..models.trailing_stop_loss_details import TrailingStopLossDetails
    from ..models.client_extensions import ClientExtensions
    from ..models.stop_loss_details import StopLossDetails


T = TypeVar("T", bound="LimitOrder")


@_attrs_define
class LimitOrder:
    """A LimitOrder is an order that is created with a price threshold, and will only be filled by a price that is equal to
    or better than the threshold.

        Attributes:
            id (Union[Unset, str]): The Order's identifier, unique within the Order's Account.
            create_time (Union[Unset, str]): The time when the Order was created.
            state (Union[Unset, LimitOrderState]): The current state of the Order.
            client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            type (Union[Unset, LimitOrderType]): The type of the Order. Always set to "LIMIT" for Limit Orders.
            instrument (Union[Unset, str]): The Limit Order's Instrument.
            units (Union[Unset, str]): The quantity requested to be filled by the Limit Order. A posititive number of units
                results in a long Order, and a negative number of units results in a short Order.
            price (Union[Unset, str]): The price threshold specified for the Limit Order. The Limit Order will only be
                filled by a market price that is equal to or better than this price.
            time_in_force (Union[Unset, LimitOrderTimeInForce]): The time-in-force requested for the Limit Order.
            gtd_time (Union[Unset, str]): The date/time when the Limit Order will be cancelled if its timeInForce is "GTD".
            position_fill (Union[Unset, LimitOrderPositionFill]): Specification of how Positions in the Account are modified
                when the Order is filled.
            trigger_condition (Union[Unset, LimitOrderTriggerCondition]): Specification of which price component should be
                used when determining if an Order should be triggered and filled. This allows Orders to be triggered based on
                the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy) price depending on
                the desired behaviour. Orders are always filled using their default price component.
                This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
                condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
                transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
                is set to the default value when indicating the distance from an Order's trigger price, and will always provide
                the default trigger condition when creating or modifying an Order.
                A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
                value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
                a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
            take_profit_on_fill (Union[Unset, TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
                Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
                a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
            stop_loss_on_fill (Union[Unset, StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
                be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
                Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
            trailing_stop_loss_on_fill (Union[Unset, TrailingStopLossDetails]): TrailingStopLossDetails specifies the
                details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
                filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
                is modified directly through the Trade.
            trade_client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            filling_transaction_id (Union[Unset, str]): ID of the Transaction that filled this Order (only provided when the
                Order's state is FILLED)
            filled_time (Union[Unset, str]): Date/time when the Order was filled (only provided when the Order's state is
                FILLED)
            trade_opened_id (Union[Unset, str]): Trade ID of Trade opened when the Order was filled (only provided when the
                Order's state is FILLED and a Trade was opened as a result of the fill)
            trade_reduced_id (Union[Unset, str]): Trade ID of Trade reduced when the Order was filled (only provided when
                the Order's state is FILLED and a Trade was reduced as a result of the fill)
            trade_closed_i_ds (Union[Unset, List[str]]): Trade IDs of Trades closed when the Order was filled (only provided
                when the Order's state is FILLED and one or more Trades were closed as a result of the fill)
            cancelling_transaction_id (Union[Unset, str]): ID of the Transaction that cancelled the Order (only provided
                when the Order's state is CANCELLED)
            cancelled_time (Union[Unset, str]): Date/time when the Order was cancelled (only provided when the state of the
                Order is CANCELLED)
            replaces_order_id (Union[Unset, str]): The ID of the Order that was replaced by this Order (only provided if
                this Order was created as part of a cancel/replace).
            replaced_by_order_id (Union[Unset, str]): The ID of the Order that replaced this Order (only provided if this
                Order was cancelled as part of a cancel/replace).
    """

    id: Union[Unset, str] = UNSET
    create_time: Union[Unset, str] = UNSET
    state: Union[Unset, LimitOrderState] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    type: Union[Unset, LimitOrderType] = UNSET
    instrument: Union[Unset, str] = UNSET
    units: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, LimitOrderTimeInForce] = UNSET
    gtd_time: Union[Unset, str] = UNSET
    position_fill: Union[Unset, LimitOrderPositionFill] = UNSET
    trigger_condition: Union[Unset, LimitOrderTriggerCondition] = UNSET
    take_profit_on_fill: Union[Unset, "TakeProfitDetails"] = UNSET
    stop_loss_on_fill: Union[Unset, "StopLossDetails"] = UNSET
    trailing_stop_loss_on_fill: Union[Unset, "TrailingStopLossDetails"] = UNSET
    trade_client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    filling_transaction_id: Union[Unset, str] = UNSET
    filled_time: Union[Unset, str] = UNSET
    trade_opened_id: Union[Unset, str] = UNSET
    trade_reduced_id: Union[Unset, str] = UNSET
    trade_closed_i_ds: Union[Unset, List[str]] = UNSET
    cancelling_transaction_id: Union[Unset, str] = UNSET
    cancelled_time: Union[Unset, str] = UNSET
    replaces_order_id: Union[Unset, str] = UNSET
    replaced_by_order_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        create_time = self.create_time

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        instrument = self.instrument

        units = self.units

        price = self.price

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        gtd_time = self.gtd_time

        position_fill: Union[Unset, str] = UNSET
        if not isinstance(self.position_fill, Unset):
            position_fill = self.position_fill

        trigger_condition: Union[Unset, str] = UNSET
        if not isinstance(self.trigger_condition, Unset):
            trigger_condition = self.trigger_condition

        take_profit_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit_on_fill, Unset):
            take_profit_on_fill = self.take_profit_on_fill.to_dict()

        stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_on_fill, Unset):
            stop_loss_on_fill = self.stop_loss_on_fill.to_dict()

        trailing_stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = self.trailing_stop_loss_on_fill.to_dict()

        trade_client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trade_client_extensions, Unset):
            trade_client_extensions = self.trade_client_extensions.to_dict()

        filling_transaction_id = self.filling_transaction_id

        filled_time = self.filled_time

        trade_opened_id = self.trade_opened_id

        trade_reduced_id = self.trade_reduced_id

        trade_closed_i_ds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.trade_closed_i_ds, Unset):
            trade_closed_i_ds = self.trade_closed_i_ds

        cancelling_transaction_id = self.cancelling_transaction_id

        cancelled_time = self.cancelled_time

        replaces_order_id = self.replaces_order_id

        replaced_by_order_id = self.replaced_by_order_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if create_time is not UNSET:
            field_dict["createTime"] = create_time
        if state is not UNSET:
            field_dict["state"] = state
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if type is not UNSET:
            field_dict["type"] = type
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if units is not UNSET:
            field_dict["units"] = units
        if price is not UNSET:
            field_dict["price"] = price
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if gtd_time is not UNSET:
            field_dict["gtdTime"] = gtd_time
        if position_fill is not UNSET:
            field_dict["positionFill"] = position_fill
        if trigger_condition is not UNSET:
            field_dict["triggerCondition"] = trigger_condition
        if take_profit_on_fill is not UNSET:
            field_dict["takeProfitOnFill"] = take_profit_on_fill
        if stop_loss_on_fill is not UNSET:
            field_dict["stopLossOnFill"] = stop_loss_on_fill
        if trailing_stop_loss_on_fill is not UNSET:
            field_dict["trailingStopLossOnFill"] = trailing_stop_loss_on_fill
        if trade_client_extensions is not UNSET:
            field_dict["tradeClientExtensions"] = trade_client_extensions
        if filling_transaction_id is not UNSET:
            field_dict["fillingTransactionID"] = filling_transaction_id
        if filled_time is not UNSET:
            field_dict["filledTime"] = filled_time
        if trade_opened_id is not UNSET:
            field_dict["tradeOpenedID"] = trade_opened_id
        if trade_reduced_id is not UNSET:
            field_dict["tradeReducedID"] = trade_reduced_id
        if trade_closed_i_ds is not UNSET:
            field_dict["tradeClosedIDs"] = trade_closed_i_ds
        if cancelling_transaction_id is not UNSET:
            field_dict["cancellingTransactionID"] = cancelling_transaction_id
        if cancelled_time is not UNSET:
            field_dict["cancelledTime"] = cancelled_time
        if replaces_order_id is not UNSET:
            field_dict["replacesOrderID"] = replaces_order_id
        if replaced_by_order_id is not UNSET:
            field_dict["replacedByOrderID"] = replaced_by_order_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.take_profit_details import TakeProfitDetails
        from ..models.trailing_stop_loss_details import TrailingStopLossDetails
        from ..models.client_extensions import ClientExtensions
        from ..models.stop_loss_details import StopLossDetails

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        create_time = d.pop("createTime", UNSET)

        _state = d.pop("state", UNSET)
        state: Union[Unset, LimitOrderState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = check_limit_order_state(_state)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        _type = d.pop("type", UNSET)
        type: Union[Unset, LimitOrderType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_limit_order_type(_type)

        instrument = d.pop("instrument", UNSET)

        units = d.pop("units", UNSET)

        price = d.pop("price", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, LimitOrderTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_limit_order_time_in_force(_time_in_force)

        gtd_time = d.pop("gtdTime", UNSET)

        _position_fill = d.pop("positionFill", UNSET)
        position_fill: Union[Unset, LimitOrderPositionFill]
        if isinstance(_position_fill, Unset):
            position_fill = UNSET
        else:
            position_fill = check_limit_order_position_fill(_position_fill)

        _trigger_condition = d.pop("triggerCondition", UNSET)
        trigger_condition: Union[Unset, LimitOrderTriggerCondition]
        if isinstance(_trigger_condition, Unset):
            trigger_condition = UNSET
        else:
            trigger_condition = check_limit_order_trigger_condition(_trigger_condition)

        _take_profit_on_fill = d.pop("takeProfitOnFill", UNSET)
        take_profit_on_fill: Union[Unset, TakeProfitDetails]
        if isinstance(_take_profit_on_fill, Unset):
            take_profit_on_fill = UNSET
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)

        _stop_loss_on_fill = d.pop("stopLossOnFill", UNSET)
        stop_loss_on_fill: Union[Unset, StopLossDetails]
        if isinstance(_stop_loss_on_fill, Unset):
            stop_loss_on_fill = UNSET
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)

        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", UNSET)
        trailing_stop_loss_on_fill: Union[Unset, TrailingStopLossDetails]
        if isinstance(_trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = UNSET
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )

        _trade_client_extensions = d.pop("tradeClientExtensions", UNSET)
        trade_client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_trade_client_extensions, Unset):
            trade_client_extensions = UNSET
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )

        filling_transaction_id = d.pop("fillingTransactionID", UNSET)

        filled_time = d.pop("filledTime", UNSET)

        trade_opened_id = d.pop("tradeOpenedID", UNSET)

        trade_reduced_id = d.pop("tradeReducedID", UNSET)

        trade_closed_i_ds = cast(List[str], d.pop("tradeClosedIDs", UNSET))

        cancelling_transaction_id = d.pop("cancellingTransactionID", UNSET)

        cancelled_time = d.pop("cancelledTime", UNSET)

        replaces_order_id = d.pop("replacesOrderID", UNSET)

        replaced_by_order_id = d.pop("replacedByOrderID", UNSET)

        limit_order = cls(
            id=id,
            create_time=create_time,
            state=state,
            client_extensions=client_extensions,
            type=type,
            instrument=instrument,
            units=units,
            price=price,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            position_fill=position_fill,
            trigger_condition=trigger_condition,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
            filling_transaction_id=filling_transaction_id,
            filled_time=filled_time,
            trade_opened_id=trade_opened_id,
            trade_reduced_id=trade_reduced_id,
            trade_closed_i_ds=trade_closed_i_ds,
            cancelling_transaction_id=cancelling_transaction_id,
            cancelled_time=cancelled_time,
            replaces_order_id=replaces_order_id,
            replaced_by_order_id=replaced_by_order_id,
        )

        limit_order.additional_properties = d
        return limit_order

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

CancellableOrderType = Literal[
    "LIMIT",
    "MARKET_IF_TOUCHED",
    "STOP",
    "STOP_LOSS",
    "TAKE_PROFIT",
    "TRAILING_STOP_LOSS",
]
CANCELLABLE_ORDER_TYPE_VALUES: Set[CancellableOrderType] = {
    "LIMIT",
    "MARKET_IF_TOUCHED",
    "STOP",
    "STOP_LOSS",
    "TAKE_PROFIT",
    "TRAILING_STOP_LOSS",
}


def check_cancellable_order_type(value: str) -> CancellableOrderType:
    if value in CANCELLABLE_ORDER_TYPE_VALUES:
        return cast(CancellableOrderType, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {CANCELLABLE_ORDER_TYPE_VALUES!r}"
    )



from __future__ import annotations
from typing import Literal, Set, cast

TrailingStopLossOrderReason = Literal["CLIENT_ORDER", "ON_FILL", "REPLACEMENT"]
TRAILING_STOP_LOSS_ORDER_REASON_VALUES: Set[TrailingStopLossOrderReason] = {
    "CLIENT_ORDER",
    "ON_FILL",
    "REPLACEMENT",
}


def check_trailing_stop_loss_order_reason(value: str) -> TrailingStopLossOrderReason:
    if value in TRAILING_STOP_LOSS_ORDER_REASON_VALUES:
        return cast(TrailingStopLossOrderReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRAILING_STOP_LOSS_ORDER_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .candlestick import Candlestick
from .get_instrument_candles_response_200_granularity import (
    GetInstrumentCandlesResponse200Granularity,
)
from .get_instrument_candles_response_200_granularity import (
    check_get_instrument_candles_response_200_granularity,
)
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="GetInstrumentCandlesResponse200")


@dataclasses.dataclass
class GetInstrumentCandlesResponse200:
    """Attributes:
    instrument (Optional[str]): The instrument whose Prices are represented by the candlesticks.
    granularity (Optional[GetInstrumentCandlesResponse200Granularity]): The granularity of the candlesticks
        provided.
    candles (Optional[List['Candlestick']]): The list of candlesticks that satisfy the request."""

    instrument: Optional[str]
    granularity: Optional[GetInstrumentCandlesResponse200Granularity]
    candles: Optional[List["Candlestick"]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .candlestick import Candlestick

        d = src_dict.copy()
        instrument = d.pop("instrument", None)
        _granularity = d.pop("granularity", None)
        granularity: Optional[GetInstrumentCandlesResponse200Granularity]
        if _granularity is None:
            granularity = None
        else:
            granularity = check_get_instrument_candles_response_200_granularity(
                _granularity
            )
        candles = []
        _candles = d.pop("candles", None)
        for candles_item_data in _candles or []:
            candles_item = Candlestick.from_dict(candles_item_data)
            candles.append(candles_item)
        get_instrument_candles_response_200 = cls(
            instrument=instrument, granularity=granularity, candles=candles
        )
        get_instrument_candles_response_200.additional_properties = d
        return get_instrument_candles_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

StopOrderReason = Literal["CLIENT_ORDER", "REPLACEMENT"]

STOP_ORDER_REASON_VALUES: Set[StopOrderReason] = {
    "CLIENT_ORDER",
    "REPLACEMENT",
}


def check_stop_order_reason(value: str) -> StopOrderReason:
    if value in STOP_ORDER_REASON_VALUES:
        return cast(StopOrderReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_ORDER_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .daily_financing_transaction_account_financing_mode import (
    DailyFinancingTransactionAccountFinancingMode,
)
from .daily_financing_transaction_account_financing_mode import (
    check_daily_financing_transaction_account_financing_mode,
)
from .daily_financing_transaction_type import DailyFinancingTransactionType
from .daily_financing_transaction_type import check_daily_financing_transaction_type
from .position_financing import PositionFinancing
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="DailyFinancingTransaction")


@dataclasses.dataclass
class DailyFinancingTransaction:
    """A DailyFinancingTransaction represents the daily payment/collection of financing for an Account.

    Attributes:
        id (Optional[str]): The Transaction's Identifier.
        time (Optional[str]): The date/time when the Transaction was created.
        user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Optional[str]): The ID of the Account the Transaction was created for.
        batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Optional[str]): The Request ID of the request which generated the transaction.
        type (Optional[DailyFinancingTransactionType]): The Type of the Transaction. Always set to "DAILY_FINANCING"
            for a DailyFinancingTransaction.
        financing (Optional[str]): The amount of financing paid/collected for the Account.
        account_balance (Optional[str]): The Account's balance after daily financing.
        account_financing_mode (Optional[DailyFinancingTransactionAccountFinancingMode]): The account financing mode
            at the time of the daily financing.
        position_financings (Optional[List['PositionFinancing']]): The financing paid/collected for each Position in
            the Account."""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[DailyFinancingTransactionType]
    financing: Optional[str]
    account_balance: Optional[str]
    account_financing_mode: Optional[DailyFinancingTransactionAccountFinancingMode]
    position_financings: Optional[List["PositionFinancing"]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .position_financing import PositionFinancing

        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[DailyFinancingTransactionType]
        if _type is None:
            type = None
        else:
            type = check_daily_financing_transaction_type(_type)
        financing = d.pop("financing", None)
        account_balance = d.pop("accountBalance", None)
        _account_financing_mode = d.pop("accountFinancingMode", None)
        account_financing_mode: Optional[DailyFinancingTransactionAccountFinancingMode]
        if _account_financing_mode is None:
            account_financing_mode = None
        else:
            account_financing_mode = (
                check_daily_financing_transaction_account_financing_mode(
                    _account_financing_mode
                )
            )
        position_financings = []
        _position_financings = d.pop("positionFinancings", None)
        for position_financings_item_data in _position_financings or []:
            position_financings_item = PositionFinancing.from_dict(
                position_financings_item_data
            )
            position_financings.append(position_financings_item)
        daily_financing_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            financing=financing,
            account_balance=account_balance,
            account_financing_mode=account_financing_mode,
            position_financings=position_financings,
        )
        daily_financing_transaction.additional_properties = d
        return daily_financing_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

AccountGuaranteedStopLossOrderMode = Literal["ALLOWED", "DISABLED", "REQUIRED"]
ACCOUNT_GUARANTEED_STOP_LOSS_ORDER_MODE_VALUES: Set[
    AccountGuaranteedStopLossOrderMode
] = {"ALLOWED", "DISABLED", "REQUIRED"}


def check_account_guaranteed_stop_loss_order_mode(
    value: str,
) -> AccountGuaranteedStopLossOrderMode:
    if value in ACCOUNT_GUARANTEED_STOP_LOSS_ORDER_MODE_VALUES:
        return cast(AccountGuaranteedStopLossOrderMode, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {ACCOUNT_GUARANTEED_STOP_LOSS_ORDER_MODE_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union

if TYPE_CHECKING:
    from ..models.market_order_reject_transaction import MarketOrderRejectTransaction


T = TypeVar("T", bound="CloseTradeResponse404")


@_attrs_define
class CloseTradeResponse404:
    """
    Attributes:
        order_reject_transaction (Union[Unset, MarketOrderRejectTransaction]): A MarketOrderRejectTransaction represents
            the rejection of the creation of a Market Order.
        last_transaction_id (Union[Unset, str]): The ID of the most recent Transaction created for the Account. Only
            present if the Account exists.
        related_transaction_i_ds (Union[Unset, List[str]]): The IDs of all Transactions that were created while
            satisfying the request. Only present if the Account exists.
        error_code (Union[Unset, str]): The code of the error that has occurred. This field may not be returned for some
            errors.
        error_message (Union[Unset, str]): The human-readable description of the error that has occurred.
    """

    order_reject_transaction: Union[Unset, "MarketOrderRejectTransaction"] = UNSET
    last_transaction_id: Union[Unset, str] = UNSET
    related_transaction_i_ds: Union[Unset, List[str]] = UNSET
    error_code: Union[Unset, str] = UNSET
    error_message: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        order_reject_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.order_reject_transaction, Unset):
            order_reject_transaction = self.order_reject_transaction.to_dict()

        last_transaction_id = self.last_transaction_id

        related_transaction_i_ds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.related_transaction_i_ds, Unset):
            related_transaction_i_ds = self.related_transaction_i_ds

        error_code = self.error_code

        error_message = self.error_message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if order_reject_transaction is not UNSET:
            field_dict["orderRejectTransaction"] = order_reject_transaction
        if last_transaction_id is not UNSET:
            field_dict["lastTransactionID"] = last_transaction_id
        if related_transaction_i_ds is not UNSET:
            field_dict["relatedTransactionIDs"] = related_transaction_i_ds
        if error_code is not UNSET:
            field_dict["errorCode"] = error_code
        if error_message is not UNSET:
            field_dict["errorMessage"] = error_message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.market_order_reject_transaction import (
            MarketOrderRejectTransaction,
        )

        d = src_dict.copy()
        _order_reject_transaction = d.pop("orderRejectTransaction", UNSET)
        order_reject_transaction: Union[Unset, MarketOrderRejectTransaction]
        if isinstance(_order_reject_transaction, Unset):
            order_reject_transaction = UNSET
        else:
            order_reject_transaction = MarketOrderRejectTransaction.from_dict(
                _order_reject_transaction
            )

        last_transaction_id = d.pop("lastTransactionID", UNSET)

        related_transaction_i_ds = cast(
            List[str], d.pop("relatedTransactionIDs", UNSET)
        )

        error_code = d.pop("errorCode", UNSET)

        error_message = d.pop("errorMessage", UNSET)

        close_trade_response_404 = cls(
            order_reject_transaction=order_reject_transaction,
            last_transaction_id=last_transaction_id,
            related_transaction_i_ds=related_transaction_i_ds,
            error_code=error_code,
            error_message=error_message,
        )

        close_trade_response_404.additional_properties = d
        return close_trade_response_404

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

MarketOrderPositionFill = Literal["DEFAULT", "OPEN_ONLY", "REDUCE_FIRST", "REDUCE_ONLY"]
MARKET_ORDER_POSITION_FILL_VALUES: Set[MarketOrderPositionFill] = {
    "DEFAULT",
    "OPEN_ONLY",
    "REDUCE_FIRST",
    "REDUCE_ONLY",
}


def check_market_order_position_fill(value: str) -> MarketOrderPositionFill:
    if value in MARKET_ORDER_POSITION_FILL_VALUES:
        return cast(MarketOrderPositionFill, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_ORDER_POSITION_FILL_VALUES!r}"
    )



from __future__ import annotations
from typing import Literal, Set, cast

Direction = Literal["LONG", "SHORT"]
DIRECTION_VALUES: Set[Direction] = {"LONG", "SHORT"}


def check_direction(value: str) -> Direction:
    if value in DIRECTION_VALUES:
        return cast(Direction, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {DIRECTION_VALUES!r}")



from typing import Literal, Set, cast

OrderFillTransactionReason = Literal[
    "LIMIT_ORDER",
    "MARKET_IF_TOUCHED_ORDER",
    "MARKET_ORDER",
    "MARKET_ORDER_DELAYED_TRADE_CLOSE",
    "MARKET_ORDER_MARGIN_CLOSEOUT",
    "MARKET_ORDER_POSITION_CLOSEOUT",
    "MARKET_ORDER_TRADE_CLOSE",
    "STOP_LOSS_ORDER",
    "STOP_ORDER",
    "TAKE_PROFIT_ORDER",
    "TRAILING_STOP_LOSS_ORDER",
]

ORDER_FILL_TRANSACTION_REASON_VALUES: Set[OrderFillTransactionReason] = {
    "LIMIT_ORDER",
    "MARKET_IF_TOUCHED_ORDER",
    "MARKET_ORDER",
    "MARKET_ORDER_DELAYED_TRADE_CLOSE",
    "MARKET_ORDER_MARGIN_CLOSEOUT",
    "MARKET_ORDER_POSITION_CLOSEOUT",
    "MARKET_ORDER_TRADE_CLOSE",
    "STOP_LOSS_ORDER",
    "STOP_ORDER",
    "TAKE_PROFIT_ORDER",
    "TRAILING_STOP_LOSS_ORDER",
}


def check_order_fill_transaction_reason(value: str) -> OrderFillTransactionReason:
    if value in ORDER_FILL_TRANSACTION_REASON_VALUES:
        return cast(OrderFillTransactionReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {ORDER_FILL_TRANSACTION_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="TradeReduce")


@_attrs_define
class TradeReduce:
    """A TradeReduce object represents a Trade for an instrument that was reduced (either partially or fully) in an
    Account. It is found embedded in Transactions that affect the position of an instrument in the account, specifically
    the OrderFill Transaction.

        Attributes:
            trade_id (Union[Unset, str]): The ID of the Trade that was reduced or closed
            units (Union[Unset, str]): The number of units that the Trade was reduced by
            price (Union[Unset, str]): The average price that the units were closed at. This price may be clamped for
                guaranteed Stop Loss Orders.
            realized_pl (Union[Unset, str]): The PL realized when reducing the Trade
            financing (Union[Unset, str]): The financing paid/collected when reducing the Trade
            guaranteed_execution_fee (Union[Unset, str]): This is the fee that is charged for closing the Trade if it has a
                guaranteed Stop Loss Order attached to it.
            half_spread_cost (Union[Unset, str]): The half spread cost for the trade reduce/close. This can be a positive or
                negative value and is represented in the home currency of the Account.
    """

    trade_id: Union[Unset, str] = UNSET
    units: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    realized_pl: Union[Unset, str] = UNSET
    financing: Union[Unset, str] = UNSET
    guaranteed_execution_fee: Union[Unset, str] = UNSET
    half_spread_cost: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        trade_id = self.trade_id

        units = self.units

        price = self.price

        realized_pl = self.realized_pl

        financing = self.financing

        guaranteed_execution_fee = self.guaranteed_execution_fee

        half_spread_cost = self.half_spread_cost

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if trade_id is not UNSET:
            field_dict["tradeID"] = trade_id
        if units is not UNSET:
            field_dict["units"] = units
        if price is not UNSET:
            field_dict["price"] = price
        if realized_pl is not UNSET:
            field_dict["realizedPL"] = realized_pl
        if financing is not UNSET:
            field_dict["financing"] = financing
        if guaranteed_execution_fee is not UNSET:
            field_dict["guaranteedExecutionFee"] = guaranteed_execution_fee
        if half_spread_cost is not UNSET:
            field_dict["halfSpreadCost"] = half_spread_cost

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        trade_id = d.pop("tradeID", UNSET)

        units = d.pop("units", UNSET)

        price = d.pop("price", UNSET)

        realized_pl = d.pop("realizedPL", UNSET)

        financing = d.pop("financing", UNSET)

        guaranteed_execution_fee = d.pop("guaranteedExecutionFee", UNSET)

        half_spread_cost = d.pop("halfSpreadCost", UNSET)

        trade_reduce = cls(
            trade_id=trade_id,
            units=units,
            price=price,
            realized_pl=realized_pl,
            financing=financing,
            guaranteed_execution_fee=guaranteed_execution_fee,
            half_spread_cost=half_spread_cost,
        )

        trade_reduce.additional_properties = d
        return trade_reduce

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .user_info_external import UserInfoExternal
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetExternalUserInfoResponse200")


@dataclasses.dataclass
class GetExternalUserInfoResponse200:
    """Attributes:
    user_info (Optional[UserInfoExternal]): A representation of user information, as available to external (3rd
        party) clients."""

    user_info: Optional["UserInfoExternal"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .user_info_external import UserInfoExternal

        d = src_dict.copy()
        _user_info = d.pop("userInfo", None)
        user_info: Optional[UserInfoExternal]
        if _user_info is None:
            user_info = None
        else:
            user_info = UserInfoExternal.from_dict(_user_info)
        get_external_user_info_response_200 = cls(user_info=user_info)
        get_external_user_info_response_200.additional_properties = d
        return get_external_user_info_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

LimitOrderTransactionType = Literal[
    "CLIENT_CONFIGURE",
    "CLIENT_CONFIGURE_REJECT",
    "CLOSE",
    "CREATE",
    "DAILY_FINANCING",
    "DELAYED_TRADE_CLOSURE",
    "FIXED_PRICE_ORDER",
    "LIMIT_ORDER",
    "LIMIT_ORDER_REJECT",
    "MARGIN_CALL_ENTER",
    "MARGIN_CALL_EXIT",
    "MARGIN_CALL_EXTEND",
    "MARKET_IF_TOUCHED_ORDER",
    "MARKET_IF_TOUCHED_ORDER_REJECT",
    "MARKET_ORDER",
    "MARKET_ORDER_REJECT",
    "ORDER_CANCEL",
    "ORDER_CANCEL_REJECT",
    "ORDER_CLIENT_EXTENSIONS_MODIFY",
    "ORDER_CLIENT_EXTENSIONS_MODIFY_REJECT",
    "ORDER_FILL",
    "REOPEN",
    "RESET_RESETTABLE_PL",
    "STOP_LOSS_ORDER",
    "STOP_LOSS_ORDER_REJECT",
    "STOP_ORDER",
    "STOP_ORDER_REJECT",
    "TAKE_PROFIT_ORDER",
    "TAKE_PROFIT_ORDER_REJECT",
    "TRADE_CLIENT_EXTENSIONS_MODIFY",
    "TRADE_CLIENT_EXTENSIONS_MODIFY_REJECT",
    "TRAILING_STOP_LOSS_ORDER",
    "TRAILING_STOP_LOSS_ORDER_REJECT",
    "TRANSFER_FUNDS",
    "TRANSFER_FUNDS_REJECT",
]
LIMIT_ORDER_TRANSACTION_TYPE_VALUES: Set[LimitOrderTransactionType] = {
    "CLIENT_CONFIGURE",
    "CLIENT_CONFIGURE_REJECT",
    "CLOSE",
    "CREATE",
    "DAILY_FINANCING",
    "DELAYED_TRADE_CLOSURE",
    "FIXED_PRICE_ORDER",
    "LIMIT_ORDER",
    "LIMIT_ORDER_REJECT",
    "MARGIN_CALL_ENTER",
    "MARGIN_CALL_EXIT",
    "MARGIN_CALL_EXTEND",
    "MARKET_IF_TOUCHED_ORDER",
    "MARKET_IF_TOUCHED_ORDER_REJECT",
    "MARKET_ORDER",
    "MARKET_ORDER_REJECT",
    "ORDER_CANCEL",
    "ORDER_CANCEL_REJECT",
    "ORDER_CLIENT_EXTENSIONS_MODIFY",
    "ORDER_CLIENT_EXTENSIONS_MODIFY_REJECT",
    "ORDER_FILL",
    "REOPEN",
    "RESET_RESETTABLE_PL",
    "STOP_LOSS_ORDER",
    "STOP_LOSS_ORDER_REJECT",
    "STOP_ORDER",
    "STOP_ORDER_REJECT",
    "TAKE_PROFIT_ORDER",
    "TAKE_PROFIT_ORDER_REJECT",
    "TRADE_CLIENT_EXTENSIONS_MODIFY",
    "TRADE_CLIENT_EXTENSIONS_MODIFY_REJECT",
    "TRAILING_STOP_LOSS_ORDER",
    "TRAILING_STOP_LOSS_ORDER_REJECT",
    "TRANSFER_FUNDS",
    "TRANSFER_FUNDS_REJECT",
}


def check_limit_order_transaction_type(value: str) -> LimitOrderTransactionType:
    if value in LIMIT_ORDER_TRANSACTION_TYPE_VALUES:
        return cast(LimitOrderTransactionType, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {LIMIT_ORDER_TRANSACTION_TYPE_VALUES!r}"
    )



from typing import Literal, Set, cast

TransferFundsRejectTransactionFundingReason = Literal[
    "ACCOUNT_TRANSFER",
    "ADJUSTMENT",
    "CLIENT_FUNDING",
    "DIVISION_MIGRATION",
    "SITE_MIGRATION",
]

TRANSFER_FUNDS_REJECT_TRANSACTION_FUNDING_REASON_VALUES: Set[
    TransferFundsRejectTransactionFundingReason
] = {
    "ACCOUNT_TRANSFER",
    "ADJUSTMENT",
    "CLIENT_FUNDING",
    "DIVISION_MIGRATION",
    "SITE_MIGRATION",
}


def check_transfer_funds_reject_transaction_funding_reason(
    value: str,
) -> TransferFundsRejectTransactionFundingReason:
    if value in TRANSFER_FUNDS_REJECT_TRANSACTION_FUNDING_REASON_VALUES:
        return cast(TransferFundsRejectTransactionFundingReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRANSFER_FUNDS_REJECT_TRANSACTION_FUNDING_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.instrument_type import check_instrument_type
from ..models.instrument_type import InstrumentType
from typing import Union

if TYPE_CHECKING:
    from ..models.instrument_commission import InstrumentCommission


T = TypeVar("T", bound="Instrument")


@_attrs_define
class Instrument:
    """Full specification of an Instrument.

    Attributes:
        name (Union[Unset, str]): The name of the Instrument
        type (Union[Unset, InstrumentType]): The type of the Instrument
        display_name (Union[Unset, str]): The display name of the Instrument
        pip_location (Union[Unset, int]): The location of the "pip" for this instrument. The decimal position of the pip
            in this Instrument's price can be found at 10 ^ pipLocation (e.g. -4 pipLocation results in a decimal pip
            position of 10 ^ -4 = 0.0001).
        display_precision (Union[Unset, int]): The number of decimal places that should be used to display prices for
            this instrument. (e.g. a displayPrecision of 5 would result in a price of "1" being displayed as "1.00000")
        trade_units_precision (Union[Unset, int]): The amount of decimal places that may be provided when specifying the
            number of units traded for this instrument.
        minimum_trade_size (Union[Unset, str]): The smallest number of units allowed to be traded for this instrument.
        maximum_trailing_stop_distance (Union[Unset, str]): The maximum trailing stop distance allowed for a trailing
            stop loss created for this instrument. Specified in price units.
        minimum_trailing_stop_distance (Union[Unset, str]): The minimum trailing stop distance allowed for a trailing
            stop loss created for this instrument. Specified in price units.
        maximum_position_size (Union[Unset, str]): The maximum position size allowed for this instrument. Specified in
            units.
        maximum_order_units (Union[Unset, str]): The maximum units allowed for an Order placed for this instrument.
            Specified in units.
        margin_rate (Union[Unset, str]): The margin rate for this instrument.
        commission (Union[Unset, InstrumentCommission]): An InstrumentCommission represents an instrument-specific
            commission
    """

    name: Union[Unset, str] = UNSET
    type: Union[Unset, InstrumentType] = UNSET
    display_name: Union[Unset, str] = UNSET
    pip_location: Union[Unset, int] = UNSET
    display_precision: Union[Unset, int] = UNSET
    trade_units_precision: Union[Unset, int] = UNSET
    minimum_trade_size: Union[Unset, str] = UNSET
    maximum_trailing_stop_distance: Union[Unset, str] = UNSET
    minimum_trailing_stop_distance: Union[Unset, str] = UNSET
    maximum_position_size: Union[Unset, str] = UNSET
    maximum_order_units: Union[Unset, str] = UNSET
    margin_rate: Union[Unset, str] = UNSET
    commission: Union[Unset, "InstrumentCommission"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        display_name = self.display_name

        pip_location = self.pip_location

        display_precision = self.display_precision

        trade_units_precision = self.trade_units_precision

        minimum_trade_size = self.minimum_trade_size

        maximum_trailing_stop_distance = self.maximum_trailing_stop_distance

        minimum_trailing_stop_distance = self.minimum_trailing_stop_distance

        maximum_position_size = self.maximum_position_size

        maximum_order_units = self.maximum_order_units

        margin_rate = self.margin_rate

        commission: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.commission, Unset):
            commission = self.commission.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if type is not UNSET:
            field_dict["type"] = type
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if pip_location is not UNSET:
            field_dict["pipLocation"] = pip_location
        if display_precision is not UNSET:
            field_dict["displayPrecision"] = display_precision
        if trade_units_precision is not UNSET:
            field_dict["tradeUnitsPrecision"] = trade_units_precision
        if minimum_trade_size is not UNSET:
            field_dict["minimumTradeSize"] = minimum_trade_size
        if maximum_trailing_stop_distance is not UNSET:
            field_dict["maximumTrailingStopDistance"] = maximum_trailing_stop_distance
        if minimum_trailing_stop_distance is not UNSET:
            field_dict["minimumTrailingStopDistance"] = minimum_trailing_stop_distance
        if maximum_position_size is not UNSET:
            field_dict["maximumPositionSize"] = maximum_position_size
        if maximum_order_units is not UNSET:
            field_dict["maximumOrderUnits"] = maximum_order_units
        if margin_rate is not UNSET:
            field_dict["marginRate"] = margin_rate
        if commission is not UNSET:
            field_dict["commission"] = commission

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.instrument_commission import InstrumentCommission

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, InstrumentType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_instrument_type(_type)

        display_name = d.pop("displayName", UNSET)

        pip_location = d.pop("pipLocation", UNSET)

        display_precision = d.pop("displayPrecision", UNSET)

        trade_units_precision = d.pop("tradeUnitsPrecision", UNSET)

        minimum_trade_size = d.pop("minimumTradeSize", UNSET)

        maximum_trailing_stop_distance = d.pop("maximumTrailingStopDistance", UNSET)

        minimum_trailing_stop_distance = d.pop("minimumTrailingStopDistance", UNSET)

        maximum_position_size = d.pop("maximumPositionSize", UNSET)

        maximum_order_units = d.pop("maximumOrderUnits", UNSET)

        margin_rate = d.pop("marginRate", UNSET)

        _commission = d.pop("commission", UNSET)
        commission: Union[Unset, InstrumentCommission]
        if isinstance(_commission, Unset):
            commission = UNSET
        else:
            commission = InstrumentCommission.from_dict(_commission)

        instrument = cls(
            name=name,
            type=type,
            display_name=display_name,
            pip_location=pip_location,
            display_precision=display_precision,
            trade_units_precision=trade_units_precision,
            minimum_trade_size=minimum_trade_size,
            maximum_trailing_stop_distance=maximum_trailing_stop_distance,
            minimum_trailing_stop_distance=minimum_trailing_stop_distance,
            maximum_position_size=maximum_position_size,
            maximum_order_units=maximum_order_units,
            margin_rate=margin_rate,
            commission=commission,
        )

        instrument.additional_properties = d
        return instrument

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
